#include "FCylinder.h"

using namespace X3DTK;

FCylinder::FCylinder()
: Cylinder(), _section(8)
{
  // Defines the tag of the node. This string must be the same than the one 
  // entered for the creation function, otherwise the node won't be loaded.
  defineTagName("FCylinder");
}

FCylinder::FCylinder(SFBool bottom, SFFloat radius, SFFloat height, SFBool side, SFBool top, SFInt32 section)
: Cylinder(bottom, radius, height, side, top), _section(section)
{
  // Defines the tag of the node.
  defineTagName("FCylinder");
}

FCylinder::FCylinder(const FCylinder &C)
: Cylinder(C), _section(C._section)
{
  // Defines the tag of the node.
  defineTagName("FCylinder");
}

SFNode FCylinder::clone() const
{
  return new FCylinder(*this);
}

void FCylinder::setSection(SFInt32 section)
{
  _section = section;
}

void FCylinder::loadAttributes(const X3DFileElement *element)
{
  Cylinder::loadAttributes(element);
  
  int index;
  index = element->getIndexAttribute("section");
  if (index != -1)
    _section = element->getAttribute(index).toInt();
}

SFString FCylinder::writeAttributes() const
{
  SFString attr;
  if (_section != 8)    
    attr += " section=\"" + toSFString(_section) + "\"";
      
  return Cylinder::writeAttributes() + attr;
}
