////////////////////////////////////////////////////////////////////////////////
//                            Color.h                                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef COLOR_H
#define COLOR_H

#include "X3DTypes.h"
#include "X3DColorNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a Color
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#Color">
/// X3D documentation</a>).

class Color : public X3DColorNode
{
public:
  /// Constructor.
  Color();
  /// Constructs a Color from its attributes.
  Color(const MFColor &color);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Color();
  
  /// Sets the Color attribute.
  void setColor(const MFColor &color);
  
  /// Gets the Color attribute.
  inline MFColor &getColor() const {return (MFColor &)color_;}; 
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected:  
  /// Copy constructor.
  Color(const Color &C);
  
private:
  MFColor color_;  
};

}

#endif
