////////////////////////////////////////////////////////////////////////////////
//                            ColorRGBA.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef COLORRGBA_H
#define COLORRGBA_H

#include "X3DTypes.h"
#include "X3DColorNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a ColorRGBA
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#ColorRGBA">
/// X3D documentation</a>).

class ColorRGBA : public X3DColorNode
{
public:
  /// Constructor.
  ColorRGBA();
  /// Constructs a ColorRGBA from its attributes.
  ColorRGBA(const MFColorRGBA &color);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~ColorRGBA();
  
  /// Sets the Color attribute.
  void setColor(const MFColorRGBA &color);
  
  /// Gets the Color attribute.
  inline MFColorRGBA &getColor() const {return (MFColorRGBA &)color_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected:
  /// Copy constructor.
  ColorRGBA(const ColorRGBA &C);
  
private:
  MFColorRGBA color_;  
};

}

#endif
