////////////////////////////////////////////////////////////////////////////////
//                            Cylinder.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef CYLINDER_H
#define CYLINDER_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Box
/// (<a href="../../doc/X3D_Specs/Part01/components/geometry3D.html#Cylinder">
/// X3D documentation</a>).

class Cylinder : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Cylinder();
  /// Constructs a Cylinder from its attributes.
  Cylinder(SFBool bottom, 
           SFFloat radius, 
	   SFFloat height, 
	   SFBool side, 
	   SFBool top);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Cylinder();
  
  /// Sets the bottom attribute.
  void setBottom(SFBool bottom);
  /// Sets the radius attribute.
  void setRadius(SFFloat radius);
  /// Sets the height attribute.
  void setHeight(SFFloat height);
  /// Sets the side attribute.
  void setSide(SFBool side);
  /// Sets the top attribute.
  void setTop(SFBool top);
  
  /// Gets the bottom attribute.
  inline SFBool getBottom() const {return bottom_;};
  /// Gets the radius attribute.
  inline SFFloat getRadius() const {return radius_;};
  /// Gets the height attribute.
  inline SFFloat getHeight() const {return height_;};
  /// Gets the side attribute.
  inline SFBool getSide() const {return side_;};
  /// Gets the top attribute.
  inline SFBool getTop() const {return top_;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.	   
  Cylinder(const Cylinder &C);

private:
  SFBool bottom_;
  SFFloat radius_;
  SFFloat height_;
  SFBool side_;
  SFBool top_;
};

}

#endif
