////////////////////////////////////////////////////////////////////////////////
//                            GLIndexedLineSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLINDEXEDLINESET_H
#define GLINDEXEDLINESET_H

#include "X3DGLGeometryNode.h"

#include <vector>

namespace X3DTK {

class IndexedLineSet;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// IndexedLineSet.

class GLIndexedLineSet : public X3DGLGeometryNode
{
public:
  /// Constructor.
  GLIndexedLineSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLIndexedLineSet();

  /// Gets the lineWidth.
  inline SFFloat &getLineWidth() const {return (SFFloat &)_lineWidth;};
  /// Gets the colorPerVertex.
  inline SFBool &getColorPerVertex() const {return (SFBool &)_colorPerVertex;};
  /// gets the color.
  inline SFBool &getColor() const {return (SFBool &)_color;};
  /// Gets the pointArray.
  inline MFVec3f &getPointArray() const {return (MFVec3f &)_pointArray;};
  /// Gets the lineArrayArray.
  inline std::vector<std::vector<C4UB_V3F> *> &getLineArrayArray() const {return (std::vector<std::vector<C4UB_V3F> *> &)_lineArrayArray;};
  /// Gets the lineIndexArray.
  inline std::vector<MFInt32 *> &getLineIndexArray() const {return (std::vector<MFInt32 *> &)_lineIndexArray;};
  /// Gets the pointLine Array.
  inline std::vector<C4UB_V3F> &getPointLineArray() const {return (std::vector<C4UB_V3F> &)_pointLineArray;};

protected:
  /// Copy constructor.      
  GLIndexedLineSet(const GLIndexedLineSet &I);
  
private:
  //GL attributes
  SFFloat _lineWidth;
  SFBool _colorPerVertex;
  SFBool _color;

  MFVec3f _pointArray;
  std::vector<std::vector<C4UB_V3F> *> _lineArrayArray;
  std::vector<MFInt32 *> _lineIndexArray;
  std::vector<C4UB_V3F> _pointLineArray;
};

}

#endif
