////////////////////////////////////////////////////////////////////////////////
//                            GLTriangleSet.h                                 //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLTRIANGLESET_H
#define GLTRIANGLESET_H

#include "X3DGLComposedGeometryNode.h"

#include <vector>

namespace X3DTK {

class TriangleSet;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// TriangleSet.

class GLTriangleSet : public X3DGLComposedGeometryNode
{
public:
  /// Constructor.
  GLTriangleSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLTriangleSet();

  /// Gets the ccw.
  inline SFBool &getCcw() const {return (SFBool &)_ccw;};
  /// Gets the solid.
  inline SFBool &getSolid() const {return (SFBool &)_solid;};
  /// Gets the color.
  inline SFBool &getColor() const {return (SFBool &)_color;};
  /// Gets the texCoord.
  inline SFBool &getTexCoord() const {return (SFBool &)_texCoord;};

  /// Gets the _N3F_V3FvertexFaceArray.
  inline std::vector<N3F_V3F> &getN3F_V3FvertexTriangleArray() const {return (std::vector<N3F_V3F> &)_N3F_V3FvertexTriangleArray;};
  /// Gets the _C4F_N3F_V3FvertexFaceArray.
  inline std::vector<C4F_N3F_V3F> &getC4F_N3F_V3FvertexTriangleArray() const {return (std::vector<C4F_N3F_V3F> &)_C4F_N3F_V3FvertexTriangleArray;};

protected:   
  /// Copy constructor.            
  GLTriangleSet(const GLTriangleSet &T);
  
private:
  //GL attributes
  SFBool _ccw;
  SFBool _solid;
  SFBool _color;
  SFBool _texCoord;
  
  std::vector<N3F_V3F> _N3F_V3FvertexTriangleArray;
  std::vector<C4F_N3F_V3F> _C4F_N3F_V3FvertexTriangleArray;
};

}

#endif
