////////////////////////////////////////////////////////////////////////////////
//                            GLUpdaterGLRenderingVisitor.h                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLUPDATERGLRENDERINGVISITOR_H
#define GLUPDATERGLRENDERINGVISITOR_H

#include "GLRenderingVisitor.h"
#include "GLUpdaterGlobalVariables.h"

namespace X3DTK {

class GLIndexedLineSet;
class GLPointSet;
class GLTriangleFanSet;
class GLTriangleSet;
class GLTriangleStripSet;

/// Visitor for the GLRendering component of the GLUpdater module.

class GLUpdaterGLRenderingVisitor : public GLRenderingVisitor
{
public:
  /// Constructor.
  GLUpdaterGLRenderingVisitor();
  /// Destructor.
  virtual ~GLUpdaterGLRenderingVisitor();

  /// Enters a GLIndexedLineSet.
  virtual void enterGLIndexedLineSet(GLIndexedLineSet *G) const;
  /// Enters a GLPointSet.
  virtual void enterGLPointSet(GLPointSet *G) const;
  /// Enters a GLTriangleFanSet.
  virtual void enterGLTriangleFanSet(GLTriangleFanSet *G) const;
  /// Enters a GLTriangleSet.
  virtual void enterGLTriangleSet(GLTriangleSet *G) const;
  /// Enters a GLTriangleStripSet.
  virtual void enterGLTriangleStripSet(GLTriangleStripSet *G) const;



protected:
  GLUpdaterGlobalVariables *globalVariables;
  
  void computeNoColor_NoTexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const;
  void computeColor_NoTexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const;
  void computeNoColor_TexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const;
  void computeColor_TexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const;
  void computeNormalArray(GLTriangleFanSet *G, MFVec3f &normalArray) const;
  
  void computeNoColor_NoTexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const;
  void computeColor_NoTexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const;
  void computeNoColor_TexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const;
  void computeColor_TexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const;
  void computeNormalArray(GLTriangleSet *G, MFVec3f &normalArray) const;
  
  void computeNoColor_NoTexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const;
  void computeColor_NoTexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const;
  void computeNoColor_TexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const;
  void computeColor_TexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const;
  void computeNormalArray(GLTriangleStripSet *G, MFVec3f &normalArray) const;
};

}

#endif
