////////////////////////////////////////////////////////////////////////////////
//                            IndexedLineSet.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef INDEXEDLINESET_H
#define INDEXEDLINESET_H

#include "X3DTypes.h"
#include "X3DGeometryNode.h"

namespace X3DTK {

class X3DColorNode;
class X3DCoordinateNode;

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining an IndexedLineSet
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#IndexedLineSet">
/// X3D documentation</a>).

class IndexedLineSet : public X3DGeometryNode
{
public:
  /// Constructor.
  IndexedLineSet();
  /// Constructs an IndexedLineSet from its attributes
  IndexedLineSet(const X3DColorNode *color,
                 const X3DCoordinateNode *coord,
                 const MFInt32 &colorIndex,
                 SFBool colorPerVertex,
                 const MFInt32 &coordIndex,
                 SFFloat lineWidth);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~IndexedLineSet();
  
  /// Sets the Color attribute.
  void setColor(const X3DColorNode *color);
  /// Sets the Coord attribute.
  void setCoord(const X3DCoordinateNode *coord);
  /// Sets the ColorIndex attribute.
  void setColorIndex(const MFInt32 &colorIndex);
  /// Sets the ColorPerVertex attribute.
  void setColorPerVertex(SFBool colorPerVertex);
  /// Sets the CoordIndex attribute.
  void setCoordIndex(const MFInt32 &coordIndex);
  /// Sets the LineWidth attribute.
  void setLineWidth(SFFloat lineWidth);
  
  /// Gets the Color attribute.
  inline SFNode getColor() const {return color_;};
  /// Gets the Coord attribute.
  inline SFNode getCoord() const {return coord_;};
  /// Gets the ColorIndex attribute.
  inline MFInt32 &getColorIndex() const {return (MFInt32 &)colorIndex_;};
  /// Gets the ColorPerVertex attribute.
  inline SFBool getColorPerVertex() const {return colorPerVertex_;};
  /// Gets the CoordIndex attribute.
  inline MFInt32 &getCoordIndex() const {return (MFInt32 &)coordIndex_;};
  /// Gets the LineWidth attribute.
  inline SFFloat getLineWidth() const {return lineWidth_;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFNode &N);
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.      
  IndexedLineSet(const IndexedLineSet &I);
  
private:
  SFNode color_;
  SFNode coord_;
  MFInt32 colorIndex_;
  SFBool colorPerVertex_;
  MFInt32 coordIndex_;
  SFFloat lineWidth_;  
};

}

#endif
