////////////////////////////////////////////////////////////////////////////////
//                            NodeWalkingProxy.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef NODEWALKINGPROXY_H
#define NODEWALKINGPROXY_H

#include "X3DNodeProxy.h"
#include "X3DNode.h"
#include "WalkingFunction.h"

#include <list>

namespace X3DTK {

class Type;
class X3DComponentWalker;
class NodeVisitingProxy;

/// Concrete class defining a NodeProxy specialized in the walking of nodes.

class NodeWalkingProxy : public X3DNodeProxy
{
public:  
  /// Constructor.
  NodeWalkingProxy();
  /// Destructor.
  virtual ~NodeWalkingProxy();
  
  /// Sets the NodeVisitingProxy.
  void setNodeVisitingProxy(const NodeVisitingProxy *nodeVisitingProxy);  
  /// Sets the X3DComponentCreator.
  void setComponentWalker(X3DComponentWalker *component);
  /// Creates the SFNode from its name.
  inline void walk(SFNode N) const {walkingArray_[N->getType()->getId()]->walk(N);};
  /// Empties the list of component.
  void reset();
  
  friend NodeWalkingProxy *joinNodeWalkingProxies(NodeWalkingProxy *N0, NodeWalkingProxy *N1);

private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  /// finds the lowest function in the derivation tree.
  WalkingFunction *getWalkingFunctionOf(const Type *type) const;
  
  //Dictionary of all the creation functions used.
  WalkingArray walkingArray_;
  std::list<X3DComponentWalker *> componentList_;
  NodeVisitingProxy *nodeVisitingProxy_;
};

}

#endif
