////////////////////////////////////////////////////////////////////////////////
//                            Normal.h                                        //
////////////////////////////////////////////////////////////////////////////////

#ifndef NORMAL_H
#define NORMAL_H

#include "X3DTypes.h"
#include "X3DNormalNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a Normal
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#Normal">
/// X3D documentation</a>).

class Normal : public X3DNormalNode
{
public:
  /// Constructor.
  Normal();
  /// Constructs a Normal from its attributes.
  Normal(const MFVec3f &vector);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Normal();
  
  /// Sets the Vector attribute.
  void setVector(const MFVec3f &vector);
  
  /// Gets the Vector attribute.
  inline MFVec3f &getVector() const {return (MFVec3f &)vector_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected: 
  /// Copy constructor.
  Normal(const Normal &N);
private:
  MFVec3f vector_;  
};

}

#endif
