////////////////////////////////////////////////////////////////////////////////
//                            PointSet.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef POINTSET_H
#define POINTSET_H

#include "X3DTypes.h"
#include "X3DGeometryNode.h"

namespace X3DTK {

class X3DColorNode;
class X3DCoordinateNode;

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a PointSet
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#PointSet">
/// X3D documentation</a>).
class PointSet : public X3DGeometryNode
{
public:
  /// Constructor.
  PointSet();
  /// Constructs a PointSet from its attributes.
  PointSet(const X3DColorNode *color,
           const X3DCoordinateNode *coord);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~PointSet();
  
  /// Sets the Color attribute.
  void setColor(const X3DColorNode *color);
  /// Sets the Coord attribute.
  void setCoord(const X3DCoordinateNode *coord);
  
  /// Gets the Color attribute.
  inline SFNode getColor() const {return color_;};
  /// Gets the Coord attribute.
  inline SFNode getCoord() const {return coord_;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFNode &N);

protected:
  /// Copy constructor. 
  PointSet(const PointSet &P);
  
private:
  SFNode color_;  
  SFNode coord_;  
};

}

#endif
