////////////////////////////////////////////////////////////////////////////////
//                            SFPoint.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef POINT3F_H
#define POINT3F_H

#include "SFVec3f.h"

#include <math.h>
#include <iostream>

namespace X3DTK {

class SFString;

/// class providing an implementation of a 3D float point. 
/// SFPoint3f is a point in an affin space, which different from a vector represented
/// by the SFVec3f class 

class SFPoint3f
{
public:
   /// x coordinates
  float x;
  /// y coordinates
  float y;
  /// z coordinates
  float z;

  /// default constructor
  SFPoint3f();
  /// constructor with x, y and z coordinates in parameters
  SFPoint3f(float x, float y, float z);
  /// copy constructor
  SFPoint3f(const SFPoint3f &p);
  /// overloaded assignment operator
  SFPoint3f &operator= (SFPoint3f v);
  /// explicit constructor from a string of which format is "x y z".
  explicit SFPoint3f(const SFString &s);
  // conversion from vector to point
  explicit SFPoint3f(const SFVec3f &V);

   /// overloaded operator+ 
  friend SFPoint3f operator+ (const SFPoint3f &v1, const SFVec3f &v2);
  /// overloaded operator- 
  friend SFPoint3f operator- (const SFPoint3f &v1, const SFVec3f &v2);
  /// overloaded operator- 
  friend SFVec3f operator- (const SFPoint3f &v1, const SFPoint3f &v2);
  /// overloaded operator* 
  friend SFPoint3f operator* (const float a, const SFPoint3f &v);
  /// norm of the point
  friend float distance(const SFPoint3f &A, const SFPoint3f &B);
};

}

#endif
