////////////////////////////////////////////////////////////////////////////////
//                            SFVec3f.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef VECTOR3F_H
#define VECTOR3F_H

#include <math.h>
#include <iostream>

namespace X3DTK {

class SFString;
class SFPoint3f;

/// class providing an implementation of a 3D float vector. 
/// SFVec3f is a vector in an affin space, which different from a point represented
/// by the SFPoint3f class 

class SFVec3f
{
public:
  /// x coordinates
  float x;
  /// y coordinates
  float y;
  /// z coordinates
  float z;

  /// default constructor
  SFVec3f();
  /// constructor with x, y and z coordinates in parameters
  SFVec3f(float x, float y, float z);
  /// copy constructor
  SFVec3f(const SFVec3f &v);
  /// overloaded assignment operator
  SFVec3f &operator= (SFVec3f v);
  /// explicit constructor from a string of which format is "x y z".
  explicit SFVec3f(const SFString &s); 
  /// conversion from point to vector
  explicit SFVec3f(const SFPoint3f &P);
  
  /// overloaded operator==
  friend bool operator== (const SFVec3f &v1, const SFVec3f &v2);
  /// overloaded operator!=
  friend bool operator!= (const SFVec3f &v1, const SFVec3f &v2);
  /// overloaded operator+
  friend SFVec3f operator+ (const SFVec3f &v1, const SFVec3f &v2);
  /// overloaded operator-
  friend SFVec3f operator- (const SFVec3f &v1, const SFVec3f &v2);
  /// overloaded operator*
  friend float operator* (const SFVec3f &v1, const SFVec3f &v2);
  /// overloaded operator*
  friend SFVec3f operator* (const float a, const SFVec3f &v);
  /// cross product between two vectors
  friend SFVec3f crossprod(const SFVec3f &v1, const SFVec3f &v2);
  /// norm of the vector
  float norm() const;
  /// normalization of the vector
  SFVec3f normalize();
  /// returns the vector normalized but doesn't change the vector
  SFVec3f normalized() const;
  /// Returns the pointed array.
  inline float *f_data() {return &x;};
};

}

#endif
