////////////////////////////////////////////////////////////////////////////////
//                            Scene.h                                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef SCENE_H
#define SCENE_H

#include "X3DTypes.h"
#include "X3DGroupingNode.h"
#include "X3DBoundedObject.h"
#include "X3DNameDict.h"

namespace X3DTK {

/// Class providing a Scene, root node of an X3D scene graph. It manages the USE/DEF.

class Scene : public X3DGroupingNode, public X3DBoundedObject
{
public:
  /// Constructor.
  Scene();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Scene();

  /// Sets the filename of the Scene.
  void setFileName(SFString file);
 
  /// Gets the name of the file.
  inline SFString getFileName() const {return file_;};
  /// Gets the DEF datas reader.
  inline X3DNameDict *getDEFDict() const {return _DEFDict;};

  /// Loads the attributes from a FileElement.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.
  Scene(const Scene &S);
  
private:
  X3DNameDict *_DEFDict;
  SFString file_;
};

}

#endif
