////////////////////////////////////////////////////////////////////////////////
//                            TriangleFanSet.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIANGLEFANSET_H
#define TRIANGLEFANSET_H

#include "X3DTypes.h"
#include "X3DComposedGeometryNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a TriangleFanSet
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#TriangleFanSet">
/// X3D documentation</a>).

class TriangleFanSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  TriangleFanSet();
  /// Constructs a triangleFanSet from its attributes.
  TriangleFanSet(const X3DColorNode *color,
                 const X3DCoordinateNode *coord,
                 const X3DNormalNode *normal,
                 const X3DTextureCoordinateNode *texCoord,
                 SFBool ccw,
                 SFBool colorPerVertex,
                 SFBool convex,
                 SFFloat creaseAngle,
                 const MFInt32 &fanCount,
                 SFBool normalPerVertex,
                 SFBool solid);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~TriangleFanSet();
  
  /// Sets the Convex attribute.
  void setConvex(SFBool convex);
  /// Sets the CreaseAngle attribute.
  void setCreaseAngle(SFFloat creaseAngle);
  /// Sets the FanCount attribute.
  void setFanCount(const MFInt32 &fanCount);
  
  /// Gets the Convex attribute.
  inline SFBool getConvex() const {return convex_;};
  /// Gets the CreaseAngle attribute.
  inline SFFloat getCreaseAngle() const {return creaseAngle_;};
  /// Gets the FanCount attribute.
  inline MFInt32 &getFanCount() const {return (MFInt32 &)fanCount_;};

  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.         
  TriangleFanSet(const TriangleFanSet &T);
  
private:
  SFBool convex_;  
  SFFloat creaseAngle_; 
  MFInt32 fanCount_; 
};

}

#endif
