////////////////////////////////////////////////////////////////////////////////
//                            TriangleStripSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIANGLESTRIPSET_H
#define TRIANGLESTRIPSET_H

#include "X3DTypes.h"
#include "X3DComposedGeometryNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a TriangleStripSet
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#TriangleStripSet">
/// X3D documentation</a>).

class TriangleStripSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  TriangleStripSet();
  /// Constructs a triangleFanSet from its attributes.
  TriangleStripSet(const X3DColorNode *color,
                   const X3DCoordinateNode *coord,
                   const X3DNormalNode *normal,
                   const X3DTextureCoordinateNode *texCoord,
                   SFBool ccw,
                   SFBool colorPerVertex,
                   SFBool convex,
                   SFFloat creaseAngle,
                   SFBool normalPerVertex,
                   SFBool solid,
                   const MFInt32 &stripCount);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~TriangleStripSet();
  
  /// Sets the Convex attribute.
  void setConvex(SFBool convex);
  /// Sets the CreaseAngle attribute.
  void setCreaseAngle(SFFloat creaseAngle);
  /// Sets the NormalPerVertex attribute.
  void setStripCount(const MFInt32 &stripCount);
  
  /// Gets the Convex attribute.
  inline SFBool getConvex() const {return convex_;};
  /// Gets the CreaseAngle attribute.
  inline SFFloat getCreaseAngle() const {return creaseAngle_;};
  /// Gets the StripCount attribute.
  inline MFInt32 &getStripCount() const {return (MFInt32 &)stripCount_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected:
  /// Copy constructor.
  TriangleStripSet(const TriangleStripSet &T);
  
private:
  SFBool convex_;  
  SFFloat creaseAngle_; 
  MFInt32 stripCount_;
};

}

#endif
