////////////////////////////////////////////////////////////////////////////////
//                            X3DGLNode.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DGLNODE_H
#define X3DGLNODE_H

#include "X3DAbstractNode.h"
#include "X3DNode.h"
#include "X3DGLTypes.h"

#include <GL/gl.h>

namespace X3DTK {

/// Base class of the Core component. A set of X3DGLNode represents a scene graph
/// related with the X3D scene graph. Each X3DNode able to be drawn directly has 
/// its X3DGLNode correspondence. For example X3DIndexedFaceSet is related with 
/// GLIndexedFaceSet, whereas Coordinate has no X3DGLNode correspondent. 

class X3DGLNode : public X3DAbstractNode
{
public:
  /// Constructor.
  X3DGLNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGLNode();
  
  /// Sets the link.
  void setLink(X3DNode *N);
  /// Returns the X3DNode equivalent.
  inline SFNode getLink() const {return link;};
  
protected:
  /// Copy constructor.
  X3DGLNode(const X3DGLNode &N);
  
  /// Pointer to the X3DNode equivalent.
  X3DNode *link;
};

}

#endif
