////////////////////////////////////////////////////////////////////////////////
//                            X3DTTAlgorithm.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DTTALGORITHM_H
#define X3DTTALGORITHM_H

#include "X3DTypes.h"

namespace X3DTK {

class NodeWalkingProxy;
class NodeVisitingProxy; 
class X3DComponentWalker;
class X3DComponentVisitor;

/// Abstract class defining the concept of tree traversal algorithm.

class X3DTTAlgorithm
{
public:
  /// Constructor.
  X3DTTAlgorithm();
  /// Destructor.  
  virtual ~X3DTTAlgorithm();
  
  /// Sets a component walker. 
  void setComponentWalker(X3DComponentWalker *component);
  /// Sets a component visitor. 
  void setComponentVisitor(X3DComponentVisitor *component);
  /// Deletes the components when the object is deleted.
  void setAutoDeleteComponents(bool value);
  /// Traverses the scene.
  void traverse(SFNode N) const;
    
protected:
  /// Visiting proxy.
  NodeVisitingProxy *nvp;     
  /// Walking proxy.
  NodeWalkingProxy *nwp;
};

}

#endif
