#include "NodeCreationProxy.h"
#include "X3DComponentCreator.h"
#include "CreationFunction.h"
#include "Type.h"
#include "Component.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

NodeCreationProxy::NodeCreationProxy()
: X3DNodeProxy() 
{
  Type::addNodeProxy(this);
}

NodeCreationProxy::~NodeCreationProxy()
{
  if (autoDelete)
    for (list<X3DComponentCreator *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      (*it)->removeOneProxy();
      if ((*it)->getProxyNumber() == 0)
        delete *it;
    }    
  
  Type::removeNodeProxy(this);
}

void NodeCreationProxy::setComponentCreator(X3DComponentCreator *component)
{
  //replace the equivalent component.
  X3DComponentCreator *replacedComponent = 0;
  bool replaced = false;
  
  for (list<X3DComponentCreator *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
  {
    if ((*it)->getName() == component->getName())
    {
      replacedComponent = *it;
      *it = component; 
      replacedComponent->removeOneProxy();
      component->addOneProxy();      
      replaced = true;
    }  
  }
  
  if (!replaced)
  {
    componentList_.push_back(component);
    component->addOneProxy();
  }
  else
  {
    // removing old functions present in the replacedComponent
    CreationDict componentCreationDict = replacedComponent->getCreationDict();
    for (CreationDict::iterator it = componentCreationDict.begin(); it != componentCreationDict.end(); ++it)
      creationDict_.erase((*it).first); 
  }
  
  //remove replaced component.
  if ((autoDelete) && (replacedComponent != 0) && (replacedComponent->getProxyNumber() == 0))
    delete replacedComponent;
    
  //adding new functions
  CreationDict componentCreationDict = component->getCreationDict();
  for (CreationDict::const_iterator it = componentCreationDict.begin(); it != componentCreationDict.end(); ++it)
      creationDict_[(*it).first] = (*it).second;
}

SFNode NodeCreationProxy::createFromName(const SFString &name) const
{
  CreationDict::const_iterator it = creationDict_.find(name);
  if (it != creationDict_.end())
    return (*it).second->create();
  
  cerr << "NodeCreationProxy::createFromName: " << name << " has no CreationFunction!" << endl;
  return 0;
}

void NodeCreationProxy::reset()
{
  //remove components
  if (autoDelete)
    for (list<X3DComponentCreator *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      (*it)->removeOneProxy();
      if ((*it)->getProxyNumber() == 0)
        delete *it;
    }    
             
  creationDict_.clear();
  componentList_.clear();
}

SFString NodeCreationProxy::getComponentNameOf(const SFString &name) const
{
  //iterating the list of component.
  for (list<X3DComponentCreator *>::const_iterator it = componentList_.begin(); it != componentList_.end(); ++it)
  {
    if ((*it)->contains(name))
      return (*it)->getName();
  }
  //not found!
  return "";
}

void NodeCreationProxy::addType(const Type *type)
{
  //finding the lowest function in the derivation tree.
  SFString name = type->getName();
  CreationFunction *CF = getCreationFunctionOf(name);
  creationDict_[name] = CF;
}

CreationFunction *NodeCreationProxy::getCreationFunctionOf(const SFString &name) const
{
  CreationFunction *CF = 0;
  
  for (list<X3DComponentCreator *>::const_iterator it = componentList_.begin(); it != componentList_.end(); ++it)
  {
    CF = (*it)->getCreationFunctionOf(name);
    if (CF != 0)
      break;
  }
  
  return CF;
}

NodeCreationProxy *X3DTK::joinNodeCreationProxy(NodeCreationProxy *N0, NodeCreationProxy *N1)
{
  NodeCreationProxy *N = new NodeCreationProxy();
  
  for (list<X3DComponentCreator *>::const_iterator it = N0->componentList_.begin(); it != N0->componentList_.end(); ++it)
    N->setComponentCreator(*it);
  
  for (list<X3DComponentCreator *>::const_iterator it = N1->componentList_.begin(); it != N1->componentList_.end(); ++it)
    N->setComponentCreator(*it);
  
  return N;  
}
