/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            NodeVisitingProxy.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef NODEVISITINGPROXY_H
#define NODEVISITINGPROXY_H

#include "X3DNodeProxy.h"

#include <list>

namespace X3DTK {

class Type;
class X3DComponentVisitor;

/// Concrete class defining a NodeProxy specialized in the visiting of nodes.

class NodeVisitingProxy : public X3DNodeProxy
{
public:  
  /// Constructor.
  NodeVisitingProxy();
  /// Destructor.
  virtual ~NodeVisitingProxy();
  
  /// Sets the X3DComponentCreator.
  void setComponentVisitor(X3DComponentVisitor *component);
  /// Enters the SFNode.
  void enter(SFNode N) const;
  /// Walks On the SFNode.
  bool walkOn(SFNode N, SFNode child = 0) const;
  /// Leaves the SFNode.
  void leave(SFNode N) const;
  /// Empties the list of component.
  void reset();
  
  friend NodeVisitingProxy *joinNodeVisitingProxies(NodeVisitingProxy *N0, NodeVisitingProxy *N1);
  
private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  /// finds the lowest functions in the derivation tree.
  EnterFunction *getEnterFunctionOf(const Type *type) const;
  WalkOnFunction *getWalkOnFunctionOf(const Type *type) const;
  LeaveFunction *getLeaveFunctionOf(const Type *type) const;
  
  //Dictionary of all the creation functions used.
  VisitingArray visitingArray_;
  std::list<X3DComponentVisitor *> componentList_;
};

}

#endif
