/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            NodeWalkingProxy.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef NODEWALKINGPROXY_H
#define NODEWALKINGPROXY_H

#include "X3DNodeProxy.h"
#include "X3DNode.h"
#include "WalkingFunction.h"

#include <list>

namespace X3DTK {

class Type;
class X3DComponentWalker;
class NodeVisitingProxy;

/// Concrete class defining a NodeProxy specialized in the walking of nodes.

class NodeWalkingProxy : public X3DNodeProxy
{
public:  
  /// Constructor.
  NodeWalkingProxy();
  /// Destructor.
  virtual ~NodeWalkingProxy();
  
  /// Sets the NodeVisitingProxy.
  void setNodeVisitingProxy(const NodeVisitingProxy *nodeVisitingProxy);  
  /// Sets the X3DComponentCreator.
  void setComponentWalker(X3DComponentWalker *component);
  /// Creates the SFNode from its name.
  inline void walk(SFNode N) const {walkingArray_[N->getType()->getId()]->walk(N);};
  /// Empties the list of component.
  void reset();
  
  friend NodeWalkingProxy *joinNodeWalkingProxies(NodeWalkingProxy *N0, NodeWalkingProxy *N1);

private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  /// finds the lowest function in the derivation tree.
  WalkingFunction *getWalkingFunctionOf(const Type *type) const;
  
  //Dictionary of all the creation functions used.
  WalkingArray walkingArray_;
  std::list<X3DComponentWalker *> componentList_;
  NodeVisitingProxy *nodeVisitingProxy_;
};

}

#endif
