/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            X3DAbstractNode.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DABSTRACTNODE_H
#define X3DABSTRACTNODE_H

#include "X3DTypes.h"
#include "X3DFileElement.h"
#include "Type.h"

namespace X3DTK {

class Scene;
class FileElement;

/// \defgroup Root Root
/// @{

/// X3DAbstractNode defines the root class of the derivation tree.

class X3DAbstractNode
{
public:
  /// Constructor.
  X3DAbstractNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DAbstractNode();

  /// Gets the name of the node.
  inline SFString getName() const {return _type->getName();};
  /// Gets the type of the node.
  inline Type *getType() const {return _type;};
  /// Gets the Scene of the node.
  inline Scene *getScene() const {return _scene;};
  /// Gets the parent list.
  inline MFNode getParentList() const {return _parentList;};
      
  /// Loads the attributes from a FileElement.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
    
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if suceeds.
  virtual bool removeChild(const SFNode &N);
 
protected:
  /// Copy constructor.
  X3DAbstractNode(const X3DAbstractNode &N);
  /// Remove link between A and B.
  void removeLink(SFNode father, SFNode child);
  /// Adds link between A and B.
  void addLink(SFNode father, SFNode child);

  /// Method to call at the first line of the body of the constructor, and passing 
  /// in argument the name of the tag of the X3D file.
  inline void defineTagName(const SFString &name, const SFString &component = SFString("")) {Type::defineTagName(&_type, name, component);};
  
private:
  // Adds a parent.
  void addParent(const SFNode &N);
  // Removes a parent.
  bool removeParent(const SFNode &N);
   
  Type *_type;
  Scene *_scene;
  MFNode _parentList;
};

/// @}

}

#endif
