#include "DefaultDFSGroupingWalker.h"
#include "NodeWalkingProxy.h"
#include "NodeVisitingProxy.h"
#include "GroupingNodes.h"

using namespace X3DTK;

DefaultDFSGroupingWalker::DefaultDFSGroupingWalker()
: DFSGroupingWalker()
{
  defineNewWalkingFunction<DefaultDFSGroupingWalker, StaticGroup>(&DefaultDFSGroupingWalker::walkStaticGroup);
  defineNewWalkingFunction<DefaultDFSGroupingWalker, X3DGroupingNode>(&DefaultDFSGroupingWalker::walkX3DGroupingNode);
}

DefaultDFSGroupingWalker::~DefaultDFSGroupingWalker()
{
}

void DefaultDFSGroupingWalker::walkStaticGroup(StaticGroup *G) const
{
  nodeVisitingProxy->enter(G);
  
  MFNode children = G->getChildren();
  for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    if ((*it != 0) && (nodeVisitingProxy->walkOn(G, *it)))
      nodeWalkingProxy->walk(*it);
  
  nodeVisitingProxy->leave(G);
}

void DefaultDFSGroupingWalker::walkX3DGroupingNode(X3DGroupingNode *N) const
{
  nodeVisitingProxy->enter(N);
  
  MFNode children = N->getChildren();
  for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    if ((*it != 0) && (nodeVisitingProxy->walkOn(N, *it)))
      nodeWalkingProxy->walk(*it);
  
  nodeVisitingProxy->leave(N);
}
