/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            CylinderDrawArray.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef CYLINDERARRAY_H
#define CYLINDERARRAY_H

#include "X3DTypes.h"
#include "X3DGLTypes.h"

#include <vector>
#include <list>
#include <utility>

namespace X3DTK {

/// Class providing the arrays required by the opengl command drawElements() to draw
/// a Cylinder. The arrays are implemented by singletons to avoid duplicating memory.

class CylinderDrawArray
{
public:
  /// Gets the CylinderDrawArray of section section.
  static CylinderDrawArray *getInstanceOfSection(unsigned int section);
  /// Removes the CylinderDrawArray of section section.
  void removeInstance();
  
  /// Gets the size of the bottom.
  unsigned int CylinderDrawArray::getCylinderBottomSize() const;
  /// Gets the size of the side.
  unsigned int CylinderDrawArray::getCylinderSideSize() const;
  /// Gets the size of the top.
  unsigned int CylinderDrawArray::getCylinderTopSize() const;
  /// Gets the address of the bottom vertex array.
  const void *CylinderDrawArray::getCylinderBottomVertexArrayAddress() const;
  /// Gets the address of the side vertex array.
  const void *CylinderDrawArray::getCylinderSideVertexArrayAddress() const;
  /// Gets the address of the top vertex array.
  const void *CylinderDrawArray::getCylinderTopVertexArrayAddress() const;
  /// Gets the address of the bottom index array.
  const unsigned int *CylinderDrawArray::getCylinderBottomIndexArrayAddress() const;
  /// Gets the address of the side index array.
  const unsigned int *CylinderDrawArray::getCylinderSideIndexArrayAddress() const;
  /// Gets the address of the top index array. 
  const unsigned int *CylinderDrawArray::getCylinderTopIndexArrayAddress() const;

private:
  unsigned int _section;
  
  CylinderDrawArray(unsigned int section); 

  typedef struct 
  {
    unsigned int count;
    CylinderDrawArray *ref;
  } data;
  
  typedef std::list<std::pair<unsigned int, data> > refList;

  static refList _refList;
  
  std::vector<N3F_V3F> _cylinderBottomVertexArray;
  std::vector<N3F_V3F> _cylinderSideVertexArray;
  std::vector<N3F_V3F>_cylinderTopVertexArray;
  std::vector<unsigned int> _cylinderBottomIndexArray;
  std::vector<unsigned int> _cylinderSideIndexArray;
  std::vector<unsigned int> _cylinderTopIndexArray;
};

}

#endif
