#include "GLAppearance.h"
#include "GLMaterial.h"
#include "X3DGLTextureNode.h"
#include "X3DGLTextureTransformNode.h"
#include "GLLineProperties.h"
#include "GLFillProperties.h"

using namespace X3DTK;

GLAppearance::GLAppearance()
: X3DGLAppearanceNode(), _material(0), _texture(0), _textureTransform(0), _lineProperties(0), _fillProperties(0)
{
  defineTagName("GLAppearance", "GLShape");
}

GLAppearance::GLAppearance(const GLAppearance &A)
: X3DGLAppearanceNode(A), _material(A._material), _texture(A._texture), _textureTransform(A._textureTransform), _lineProperties(A._lineProperties), _fillProperties(A._fillProperties)
{
  defineTagName("GLAppearance", "GLShape");
}

SFNode GLAppearance::clone() const
{
  return new GLAppearance(*this);
}

GLAppearance::~GLAppearance()
{
}

void GLAppearance::setMaterial(const GLMaterial *material) 
{
  _material = (GLMaterial *)material;
}
  
void GLAppearance::setTexture(const X3DGLTextureNode *texture) 
{
  _texture = (X3DGLTextureNode *)texture;
}

void GLAppearance::setTextureTransform(const X3DGLTextureTransformNode *textureTransform) 
{
  _textureTransform = (X3DGLTextureTransformNode *)textureTransform;
}

void GLAppearance::setLineProperties(const GLLineProperties *lineProperties) 
{
  _lineProperties = (GLLineProperties *)lineProperties;
}

void GLAppearance::setFillProperties(const GLFillProperties *fillProperties) 
{
  _fillProperties = (GLFillProperties *)fillProperties;
}

bool GLAppearance::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DGLMaterialNode *>(N) != 0)
  {
    if (_material != 0)
      return false;
      
    _material = N;
    return true;
  }
  
  if (dynamic_cast<X3DGLTextureNode *>(N) != 0)
  {
    if (_texture != 0)
      return false;
      
    _texture = N;
    return true;    
  }
  
  if (dynamic_cast<X3DGLTextureTransformNode *>(N) != 0)
  {
    if (_textureTransform != 0)
      return false;
      
    _textureTransform = N;
    return true;    
  }
  
  if (dynamic_cast<GLLineProperties *>(N) != 0)
  {
    if (_lineProperties != 0)
      return false;
      
    _lineProperties = N;
    return true;    
  }
  
  if (dynamic_cast<GLFillProperties *>(N) != 0)
  {
    if (_fillProperties != 0)
      return false;
      
    _fillProperties = N;
    return true;    
  }
  
  return false;
}

bool GLAppearance::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DGLMaterialNode *>(N) != 0)
  {
    _material = N;
    return true;
  }
  
  if (dynamic_cast<X3DGLTextureNode *>(N) != 0)
  {
    _texture = N;
    return true;    
  }
  
  if (dynamic_cast<X3DGLTextureTransformNode *>(N) != 0)
  {
    _textureTransform = N;
    return true;    
  }
  
  if (dynamic_cast<GLLineProperties *>(N) != 0)
  {
    _lineProperties = N;
    return true;    
  }
  
  if (dynamic_cast<GLFillProperties *>(N) != 0)
  {
    _fillProperties = N;
    return true;    
  }
  
  return false;
}

bool GLAppearance::removeChild(const SFNode &N)
{
  if (_material == N)
  {
    _material = 0;
    return true;
  }
  
  if (_texture == N)
  {
    _texture = 0;
    return true; 
  }
  
  if (_textureTransform == N)
  {
    _textureTransform = 0;
    return true;  
  }
  
  if (_lineProperties == N)
  {
    _lineProperties = 0;
    return true;
  }
  
  if (_fillProperties == N)
  {
    _fillProperties = 0;
    return true; 
  }
  
  return false;
}
