#include "Cone.h"

using namespace X3DTK;

Cone::Cone()
: X3DGeometry3DNode(), bottomRadius_(1.0f), height_(2.0f), side_(true), bottom_(true)  
{
  defineTagName("Cone", "Geometry3D");
}

Cone::Cone(SFFloat bottomRadius, SFFloat height, SFBool side, SFBool bottom)
: X3DGeometry3DNode(), bottomRadius_(bottomRadius), height_(height), side_(side), bottom_(bottom)
{
  defineTagName("Cone", "Geometry3D");
}

Cone::Cone(const Cone &C)
: X3DGeometry3DNode(C), bottomRadius_(C.bottomRadius_), height_(C.height_), side_(C.side_), bottom_(C.bottom_)
{
  defineTagName("Cone", "Geometry3D");
}

SFNode Cone::clone() const
{
  return new Cone(*this);
}

Cone::~Cone()
{
}

void Cone::setBottomRadius(SFFloat bottomRadius)
{
  bottomRadius_ = bottomRadius;
}

void Cone::setHeight(SFFloat height)
{
  height_ = height;
}

void Cone::setSide(SFBool side)
{
  side_ = side;
}

void Cone::setBottom(SFBool bottom)
{
  bottom_ = bottom;
}

void Cone::loadAttributes(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("bottomRadius");
  if (index != -1)
    bottomRadius_ = element->getAttribute(index).toFloat(); 

  index = element->getIndexAttribute("height");
  if (index != -1)
    height_ = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("side");
  if (index != -1)
    side_ = (element->getAttribute(index).upper() == "TRUE"); 
  
  index = element->getIndexAttribute("bottom");
  if (index != -1)
    bottom_ = (element->getAttribute(index).upper() == "TRUE");
}

SFString Cone::writeAttributes() const
{
  SFString attr;
  if (bottomRadius_ != 1.0f)
    attr += " bottomRadius=\"" + toSFString(bottomRadius_) + "\"";
  if (height_ != 2.0f)  
    attr += " height=\"" + toSFString(height_) + "\"";
  if (!side_)  
    attr += " side=\"FALSE\"";
  if (!bottom_)  
    attr += " bottom=\"FALSE\"";
      
  return attr;
}
