#include "StaticGroup.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

StaticGroup::StaticGroup()
: X3DChildNode(), X3DBoundedObject()
{
  defineTagName("StaticGroup", "Grouping");
}

StaticGroup::StaticGroup(const SFVec3f &bboxCenter, const SFVec3f &bboxSize)
: X3DChildNode(), X3DBoundedObject(bboxCenter, bboxSize)
{
  defineTagName("StaticGroup", "Grouping");
}

StaticGroup::StaticGroup(const StaticGroup &G)
: X3DChildNode(G), X3DBoundedObject(G), children_(G.children_)
{
  defineTagName("StaticGroup", "Grouping");
}

SFNode StaticGroup::clone() const
{
  return new StaticGroup(*this);
}

StaticGroup::~StaticGroup()
{
}

bool StaticGroup::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    children_.push_back(N);
    addLink(this, N);
    return true;
  }
  
  cerr << "StaticGroup::addChild : a node of type " << N->getName() << " cannot be a child!" << endl;
  return false;    
}

bool StaticGroup::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    children_.push_back(N);
    addLink(this, N);
    return true;
  }
  
  cerr << "StaticGroup::setChild : a node of type " << N->getName() << " cannot be a child!" << endl;
  return false;    
}

bool StaticGroup::removeChild(const SFNode &N)
{
  MFNode::iterator res = find(children_.begin(), children_.end(), N);
  if (res != children_.end())
  {
    children_.erase(res);
    removeLink(this, N);
    return true;
  }
      
  return false;
}

void StaticGroup::loadAttributes(const X3DFileElement *element)
{
  X3DBoundedObject::loadAttributes(element);
}

SFString StaticGroup::writeAttributes() const
{
  return X3DBoundedObject::writeAttributes();
}
