#include "Transform.h"

using namespace X3DTK;

Transform::Transform()
: X3DGroupingNode(), X3DBoundedObject(), center_(SFVec3f(0.0f, 0.0f, 0.0f)), rotation_(SFRotation(0.0f, 0.0f, 1.0f, 0.0f)), scale_(SFVec3f(1.0f, 1.0f, 1.0f)), scaleOrientation_(SFRotation(0.0f, 0.0f, 1.0f, 0.0f)), translation_(SFVec3f(0.0f, 0.0f, 0.0f))
{
  defineTagName("Transform", "Grouping");
}

Transform::Transform(const SFVec3f &center, const SFRotation &rotation, const SFVec3f &scale,  const SFRotation &scaleOrientation,  const SFVec3f &translation,  const SFVec3f &bboxCenter,  const SFVec3f &bboxSize)
: X3DGroupingNode(), X3DBoundedObject(bboxCenter, bboxSize), center_(center), rotation_(rotation), scale_(scale), scaleOrientation_(scaleOrientation), translation_(translation)
{
  defineTagName("Transform", "Grouping");
}

Transform::Transform(const Transform &T)
: X3DGroupingNode(T), X3DBoundedObject(T), center_(T.center_), rotation_(T.rotation_), scale_(T.scale_), scaleOrientation_(T.scaleOrientation_), translation_(T.translation_)
{
  defineTagName("Transform", "Grouping");
}

SFNode Transform::clone() const
{
  return new Transform(*this);
}

Transform::~Transform()
{
}

void Transform::setCenter(const SFVec3f &center)
{
  center_ = center;
}

void Transform::setRotation(const SFRotation &rotation)
{
  rotation_ = rotation;
}

void Transform::setScale(const SFVec3f &scale)
{
  scale_ = scale;
}

void Transform::setScaleOrientation(const SFRotation &scaleOrientation)
{
  scaleOrientation_ = scaleOrientation;
}

void Transform::setTranslation(const SFVec3f &translation)
{
  translation_ = translation;
}

void Transform::loadAttributes(const X3DFileElement *element)
{
  X3DBoundedObject::loadAttributes(element);
  
  int index;  
  index = element->getIndexAttribute("center");
  if (index != -1)
    center_ = SFVec3f(element->getAttribute(index)); 
  
  index = element->getIndexAttribute("rotation"); 
  if (index != -1)
    rotation_ = SFRotation(element->getAttribute(index)); 
  
  index = element->getIndexAttribute("scale");
  if (index != -1)
    scale_ = SFVec3f(element->getAttribute(index)); 
 
  index = element->getIndexAttribute("scaleOrientation"); 
  if (index != -1)
    scaleOrientation_ = SFRotation(element->getAttribute(index)); 
   
  index = element->getIndexAttribute("translation"); 
  if (index != -1)
    translation_ = SFVec3f(element->getAttribute(index)); 
}

SFString Transform::writeAttributes() const
{
  SFString attr;
  if (center_ != SFVec3f(0.0f, 0.0f, 0.0f))
    attr += " center=\"" + toSFString(center_) + "\"";
  if (rotation_ != SFRotation(0.0f, 0.0f, 1.0f, 0.0f))  
    attr += " rotation=\"" + toSFString(rotation_) + "\"";
  if (scale_ != SFVec3f(1.0f, 1.0f, 1.0f))
    attr += " scale=\"" + toSFString(scale_) + "\"";
  if (scaleOrientation_ != SFRotation(0.0f, 0.0f, 1.0f, 0.0f))  
    attr += " scaleOrientation=\"" + toSFString(scaleOrientation_) + "\"";
  if (translation_ != SFVec3f(0.0f, 0.0f, 0.0f))
    attr += " translation=\"" + toSFString(translation_) + "\"";
 
  return attr;   
}
