/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            X3DBoundedObject.h                              //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DBOUNDEDOBJECT_H
#define X3DBOUNDEDOBJECT_H

#include "X3DTypes.h"
#include "X3DObject.h"
#include "X3DFileElement.h"

namespace X3DTK {

/// Base class of the <a href="../../doc/X3D_Specs/Part01/components/group.html">
/// Grouping component</a> defining an X3DBoundedObject
/// (<a href="../../doc/X3D_Specs/Part01/components/grouping.html#X3DBoundedObject">
/// X3D documentation</a>).

class X3DBoundedObject : public X3DObject
{
public:
  /// Constructor.
  X3DBoundedObject();
  /// Constructs an X3DBoundedObject from its attributes.
  X3DBoundedObject(const SFVec3f &bboxCenter,
                   const SFVec3f &bboxSize);
		   
  /// Copy constructor.
  X3DBoundedObject(const X3DBoundedObject &N);

  /// Virtual destructor.
  virtual ~X3DBoundedObject();

  /// Sets the BboxCenter attribute.
  void setBboxCenter(const SFVec3f &bboxCenter);
  /// Sets the BboxSize attribute.
  void setBboxSize(const SFVec3f &bboxSize);

  /// Gets the BboxCenter attribute.
  inline SFVec3f getBboxCenter() const {return bboxCenter_;};
  /// Gets the BboxSize attribute.
  inline SFVec3f getBboxSize() const {return bboxSize_;};

  /// Loads the attributes from a X3DFileElement e.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Center of the Bounding Box.
  SFVec3f bboxCenter_;
  /// Size of the Bounding Box.
  SFVec3f bboxSize_; 
};

}

#endif
