/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            X3DGroupingNode.h                               //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DGROUPINGNODE
#define X3DGROUPINGNODE

#include "X3DTypes.h"
#include "X3DChildNode.h"

namespace X3DTK {

/// Base class of the <a href="../../doc/X3D_Specs/Part01/components/group.html">
/// Grouping component</a> defining an X3DGroupingNode
/// (<a href="../../doc/X3D_Specs/Part01/components/grouping.html#X3DGroupingNode">
/// X3D documentation</a>).

class X3DGroupingNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DGroupingNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGroupingNode();

  /// Adds a child to the Children attribute. Return TRUE if succeeds.
  bool addChild(const SFNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return children_;};
  
protected:
  /// Copy constructor.		  
  X3DGroupingNode(const X3DGroupingNode &N);
  
  /// Children attribute.
  MFNode children_;
};

}

#endif
