#include "IndexedLineSet.h"
#include "RenderingNodes.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

IndexedLineSet::IndexedLineSet()
: X3DGeometryNode(), color_(0), coord_(0), colorPerVertex_(true), lineWidth_(1.0f)
{
  defineTagName("IndexedLineSet", "Rendering");
}

IndexedLineSet::IndexedLineSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const MFInt32 &colorIndex, SFBool colorPerVertex, const MFInt32 &coordIndex, SFFloat lineWidth)
: X3DGeometryNode(), color_((SFNode)color), coord_((SFNode)coord), colorIndex_(colorIndex), colorPerVertex_(colorPerVertex), coordIndex_(coordIndex), lineWidth_(lineWidth)
{
  defineTagName("IndexedLineSet", "Rendering");
}

IndexedLineSet::IndexedLineSet(const IndexedLineSet &I)
: X3DGeometryNode(), color_(I.color_), coord_(I.coord_), colorIndex_(I.colorIndex_), colorPerVertex_(I.colorPerVertex_), coordIndex_(I.coordIndex_), lineWidth_(I.lineWidth_)
{
  defineTagName("IndexedLineSet", "Rendering");
}  

SFNode IndexedLineSet::clone() const
{
  return new IndexedLineSet(*this);
}

IndexedLineSet::~IndexedLineSet()
{
}  
  
//modifiers
void IndexedLineSet::setColor(const X3DColorNode *color)
{
  removeLink(this, color_);
  color_ = (SFNode)color;
  addLink(this, color_);
}

void IndexedLineSet::setCoord(const X3DCoordinateNode *coord)
{
  removeLink(this, coord_);  
  coord_ = (SFNode)coord;
  addLink(this, coord_);
}

void IndexedLineSet::setColorIndex(const MFInt32 &colorIndex)
{
  colorIndex_ = colorIndex;
}

void IndexedLineSet::setColorPerVertex(SFBool colorPerVertex)
{
  colorPerVertex_ = colorPerVertex;
}

void IndexedLineSet::setCoordIndex(const MFInt32 &coordIndex)
{
  coordIndex_ = coordIndex;
  if ((!coordIndex_.empty()) && (coordIndex_.back() != -1))
    coordIndex_.push_back(-1);
}

void IndexedLineSet::setLineWidth(SFFloat lineWidth)
{
  lineWidth_ = lineWidth;
}
  
bool IndexedLineSet::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    if (color_ != 0)
      return false;
      
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    if (coord_ != 0)
      return false;
      
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "IndexedLineSet::addChild : a node of type " << N->getName() << " cannot be a child!" << endl;
  return false;
}

bool IndexedLineSet::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "IndexedLineSet::setChild : a node of type " << N->getName() << " cannot be a child!" << endl;
  return false;
}

bool IndexedLineSet::removeChild(const SFNode &N)
{
  if (color_ == N)
  {
    setColor(0);
    return true;
  } 
   
  if (coord_ == N)
  {
    setCoord(0);
    return true;
  } 
  
  return false;
}

void IndexedLineSet::loadAttributes(const X3DFileElement *element)
{ 
  int index;
  index = element->getIndexAttribute("colorIndex");
  if (index != -1)
    colorIndex_ = MFInt32(element->getAttribute(index));  
  
  index = element->getIndexAttribute("colorPerVertex");
  if (index != -1)
    colorPerVertex_ = (element->getAttribute(index).upper() == "TRUE");
  
  index = element->getIndexAttribute("coordIndex");  
  if (index != -1)
    coordIndex_ = MFInt32(element->getAttribute(index));
    
  index = element->getIndexAttribute("lineWidth");    
  if (index != -1)
    lineWidth_ = element->getAttribute(index).toFloat();
}

SFString IndexedLineSet::writeAttributes() const
{
  SFString attr;
  if (!colorIndex_.empty())
    attr += " colorIndex=\"" + toSFString(colorIndex_) + "\"";
  if (!colorPerVertex_)
    attr += " colorPerVertex=\"FALSE\"";
  if (!coordIndex_.empty())  
    attr += " coordIndex=\"" + toSFString(coordIndex_) + "\"";
  if (lineWidth_ != 1.0f)
    attr += " lineWidth=\"" + toSFString(lineWidth_) + "\"";
    
  return attr;
}
