/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            IndexedLineSet.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef INDEXEDLINESET_H
#define INDEXEDLINESET_H

#include "X3DTypes.h"
#include "X3DGeometryNode.h"

namespace X3DTK {

class X3DColorNode;
class X3DCoordinateNode;

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining an IndexedLineSet
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#IndexedLineSet">
/// X3D documentation</a>).

class IndexedLineSet : public X3DGeometryNode
{
public:
  /// Constructor.
  IndexedLineSet();
  /// Constructs an IndexedLineSet from its attributes
  IndexedLineSet(const X3DColorNode *color,
                 const X3DCoordinateNode *coord,
                 const MFInt32 &colorIndex,
                 SFBool colorPerVertex,
                 const MFInt32 &coordIndex,
                 SFFloat lineWidth);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~IndexedLineSet();
  
  /// Sets the Color attribute.
  void setColor(const X3DColorNode *color);
  /// Sets the Coord attribute.
  void setCoord(const X3DCoordinateNode *coord);
  /// Sets the ColorIndex attribute.
  void setColorIndex(const MFInt32 &colorIndex);
  /// Sets the ColorPerVertex attribute.
  void setColorPerVertex(SFBool colorPerVertex);
  /// Sets the CoordIndex attribute.
  void setCoordIndex(const MFInt32 &coordIndex);
  /// Sets the LineWidth attribute.
  void setLineWidth(SFFloat lineWidth);
  
  /// Gets the Color attribute.
  inline SFNode getColor() const {return color_;};
  /// Gets the Coord attribute.
  inline SFNode getCoord() const {return coord_;};
  /// Gets the ColorIndex attribute.
  inline MFInt32 &getColorIndex() const {return (MFInt32 &)colorIndex_;};
  /// Gets the ColorPerVertex attribute.
  inline SFBool getColorPerVertex() const {return colorPerVertex_;};
  /// Gets the CoordIndex attribute.
  inline MFInt32 &getCoordIndex() const {return (MFInt32 &)coordIndex_;};
  /// Gets the LineWidth attribute.
  inline SFFloat getLineWidth() const {return lineWidth_;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFNode &N);
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.      
  IndexedLineSet(const IndexedLineSet &I);
  
private:
  SFNode color_;
  SFNode coord_;
  MFInt32 colorIndex_;
  SFBool colorPerVertex_;
  MFInt32 coordIndex_;
  SFFloat lineWidth_;  
};

}

#endif
