#include "TriangleFanSet.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

TriangleFanSet::TriangleFanSet()
: X3DComposedGeometryNode(), convex_(true), creaseAngle_(0.0f)
{
  defineTagName("TriangleFanSet", "Rendering");
}

TriangleFanSet::TriangleFanSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool convex, SFFloat creaseAngle, const MFInt32 &fanCount, SFBool normalPerVertex, SFBool solid)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
convex_(convex), creaseAngle_(creaseAngle), fanCount_(fanCount)
{
  defineTagName("TriangleFanSet", "Rendering");
}

           
TriangleFanSet::TriangleFanSet(const TriangleFanSet &T)
: X3DComposedGeometryNode(T), convex_(T.convex_), creaseAngle_(T.creaseAngle_), fanCount_(T.fanCount_)
{
  defineTagName("TriangleFanSet", "Rendering");
}

SFNode TriangleFanSet::clone() const
{
  return new TriangleFanSet(*this);
}

TriangleFanSet::~TriangleFanSet()
{
}

void TriangleFanSet::setConvex(SFBool convex)
{
  convex_ = convex;
}

void TriangleFanSet::setCreaseAngle(SFFloat creaseAngle)
{
  creaseAngle_ = creaseAngle;
}

void TriangleFanSet::setFanCount(const MFInt32 &fanCount)
{
  fanCount_  = fanCount;
}

void TriangleFanSet::loadAttributes(const X3DFileElement *element)
{
  X3DComposedGeometryNode::loadAttributes(element);
  
  int index;
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    convex_ = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    creaseAngle_ = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("fanCount");
  if (index != -1)
    fanCount_ = MFInt32(element->getAttribute(index));  
}

SFString TriangleFanSet::writeAttributes() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::writeAttributes();
  
  if (!convex_)  
    attr += " convex=\"FALSE\"";
  if (creaseAngle_ != 0.0f)
    attr += " creaseAngle=\"" + toSFString(creaseAngle_) + "\"";
  if (!fanCount_.empty())
    attr += " fanCount=\"" + toSFString(fanCount_) + "\"";
  
  return attr;
}
