#include "TriangleSet.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

TriangleSet::TriangleSet()
: X3DComposedGeometryNode(), convex_(true), creaseAngle_(0.0f)
{
  defineTagName("TriangleSet", "Rendering");
}

TriangleSet::TriangleSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool convex, SFFloat creaseAngle, SFBool normalPerVertex, SFBool solid)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
convex_(convex), creaseAngle_(creaseAngle)
{
  defineTagName("TriangleSet", "Rendering");
}

TriangleSet::TriangleSet(const TriangleSet &T)
: X3DComposedGeometryNode(T), convex_(T.convex_), creaseAngle_(T.creaseAngle_)
{
  defineTagName("TriangleSet", "Rendering");
}

SFNode TriangleSet::clone() const
{
  return new TriangleSet(*this);
}

TriangleSet::~TriangleSet()
{
}

void TriangleSet::setConvex(SFBool convex)
{
  convex_ = convex;
}

void TriangleSet::setCreaseAngle(SFFloat creaseAngle)
{
  creaseAngle_ = creaseAngle;
}

void TriangleSet::loadAttributes(const X3DFileElement *element)
{
  X3DComposedGeometryNode::loadAttributes(element);

  int index;  
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    convex_ = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    creaseAngle_ = element->getAttribute(index).toFloat(); 
}    
    
SFString TriangleSet::writeAttributes() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::writeAttributes();
  
  if (!convex_)  
    attr += " convex=\"FALSE\"";
  if (creaseAngle_ != 0.0f)
    attr += " creaseAngle=\"" + toSFString(creaseAngle_) + "\"";
    
  return attr;
}
