#include "X3DComposedGeometryNode.h"
#include "RenderingNodes.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

X3DComposedGeometryNode::X3DComposedGeometryNode()
: X3DGeometry3DNode(), color_(0), coord_(0), normal_(0), texCoord_(0), ccw_(true), colorPerVertex_(true), normalPerVertex_(true), solid_(true)
{
  defineTagName("X3DComposedGeometryNode");
}

X3DComposedGeometryNode::X3DComposedGeometryNode(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool normalPerVertex, SFBool solid)
: X3DGeometry3DNode(), color_((SFNode)color), coord_((SFNode)coord), normal_((SFNode)normal), texCoord_((SFNode)texCoord), ccw_(ccw), colorPerVertex_(colorPerVertex), normalPerVertex_(normalPerVertex), solid_(solid)
{
  defineTagName("X3DComposedGeometryNode");
}

           
X3DComposedGeometryNode::X3DComposedGeometryNode(const X3DComposedGeometryNode &N)
: X3DGeometry3DNode(N), color_(N.color_), coord_(N.coord_), normal_(N.normal_), texCoord_(N.texCoord_), ccw_(N.ccw_), colorPerVertex_(N.colorPerVertex_), normalPerVertex_(N.normalPerVertex_), solid_(N.solid_)
{
  defineTagName("X3DComposedGeometryNode");
}

SFNode X3DComposedGeometryNode::clone() const
{
  return 0;
}

X3DComposedGeometryNode::~X3DComposedGeometryNode()
{
}

void X3DComposedGeometryNode::setColor(const X3DColorNode *color)
{
  removeLink(this, color_);
  color_ = (SFNode)color;
  addLink(this, color_);
}

void X3DComposedGeometryNode::setCoord(const X3DCoordinateNode *coord)
{
  removeLink(this, coord_);
  coord_ = (SFNode)coord;
  addLink(this, coord_);
}

void X3DComposedGeometryNode::setNormal(const X3DNormalNode *normal)
{
  removeLink(this, normal_);
  normal_ = (SFNode)normal;
  addLink(this, normal_);
}

void X3DComposedGeometryNode::setTexCoord(const X3DTextureCoordinateNode *texCoord)
{
  removeLink(this, texCoord_);
  texCoord_ = (SFNode)texCoord;
  addLink(this, texCoord_);
}

void X3DComposedGeometryNode::setCcw(SFBool ccw)
{
  ccw_= ccw;
}

void X3DComposedGeometryNode::setColorPerVertex(SFBool colorPerVertex)
{
  colorPerVertex_ = colorPerVertex;
}

void X3DComposedGeometryNode::setNormalPerVertex(SFBool normalPerVertex)
{
  normalPerVertex_ = normalPerVertex;
}

void X3DComposedGeometryNode::setSolid(SFBool solid)
{
  solid_ = solid;
}

bool X3DComposedGeometryNode::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    if (color_ != 0)
      return false;
    
    setColor(static_cast<X3DColorNode *>(N)); 
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    if (coord_ != 0)
      return false;
      
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DNormalNode *>(N) != 0)
  {
    if (normal_ != 0)
      return false;
      
    setNormal(static_cast<X3DNormalNode *>(N));   
    return true;
  }
  
  if (dynamic_cast<X3DNode *>(N) != 0)
  {
    if (texCoord_ != 0)
      return false;
    
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));   
    return true;    
  }
  return false;
}

bool X3DComposedGeometryNode::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    setColor(static_cast<X3DColorNode *>(N)); 
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DNormalNode *>(N) != 0)
  {
    setNormal(static_cast<X3DNormalNode *>(N));  
    return true;
  }
  
  if (dynamic_cast<X3DTextureCoordinateNode *>(N) != 0)
  {
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "X3DComposedGeometryNode::addChild : a node of type " << N->getName() << " cannot be a child!" << endl;
  return false;
}

bool X3DComposedGeometryNode::removeChild(const SFNode &N)
{
  if (color_ == N)
  {
    setColor(0);
    return true;
  } 
   
  if (coord_ == N)
  {
    setCoord(0);
    return true;
  } 

  if (normal_ == N)
  {
    setNormal(0);
    return true;
  } 
   
  if (texCoord_ == N)
  {
    setTexCoord(0);
    return true;
  } 
  
  cerr << "X3DComposedGeometryNode::removeChild : a node of type " << N->getName() << " cannot be a child!" << endl;
  return false;
}

void X3DComposedGeometryNode::loadAttributes(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("ccw");
  if (index != -1)
    ccw_ = (element->getAttribute(index).upper() == "TRUE");
  
  index = element->getIndexAttribute("colorPerVertex");  
  if (index != -1)
    colorPerVertex_ = (element->getAttribute(index).upper() == "TRUE");  

  index = element->getIndexAttribute("normalPerVertex");
  if (index != -1)
    normalPerVertex_ = (element->getAttribute(index).upper() == "TRUE");  
  
  index = element->getIndexAttribute("solid");  
  if (index != -1)
    solid_ = (element->getAttribute(index).upper() == "TRUE");
}

SFString X3DComposedGeometryNode::writeAttributes() const
{
  SFString attr;
  if (!ccw_)
    attr += " ccw=\"FALSE\"";
  if (!colorPerVertex_)  
    attr += " colorPerVertex=\"FALSE\"";
  if (!normalPerVertex_)
    attr += " normalPerVertex=\"FALSE\"";
  if (!solid_)  
    attr += " solid=\"FALSE\"";  
  
  return attr;
}
