#include "Appearance.h"
#include "ShapeNodes.h"

using namespace X3DTK;

Appearance::Appearance()
: X3DAppearanceNode(), material_(0), texture_(0), textureTransform_(0), lineProperties_(0), fillProperties_(0)
{
  defineTagName("Appearance", "Shape");
}

Appearance::Appearance(const X3DMaterialNode *material,  const X3DTextureNode *texture, const X3DTextureTransformNode *textureTransform, const LineProperties *lineProperties, const FillProperties *fillProperties)
: X3DAppearanceNode(), material_((SFNode)material), texture_((SFNode)texture), textureTransform_((SFNode)textureTransform), lineProperties_((SFNode)lineProperties), fillProperties_((SFNode)fillProperties)
{
  defineTagName("Appearance", "Shape");
}

Appearance::Appearance(const Appearance &A)
: X3DAppearanceNode(A), material_(A.material_), texture_(A.texture_), textureTransform_(A.textureTransform_), lineProperties_(A.lineProperties_), fillProperties_(A.fillProperties_)
{
  defineTagName("Appearance", "Shape");
}

SFNode Appearance::clone() const
{
  return new Appearance(*this);
}

Appearance::~Appearance()
{
}

void Appearance::setMaterial(const X3DMaterialNode *material)
{
  removeLink(this, material_);
  material_ = (SFNode)material;
  addLink(this, material_);
}

void Appearance::setTexture(const X3DTextureNode *texture)
{
  removeLink(this, texture_);
  texture_ = (SFNode)texture;
  addLink(this, texture_);
}

void Appearance::setTextureTransform(const X3DTextureTransformNode *textureTransform)
{
  removeLink(this, textureTransform_);
  textureTransform_ = (SFNode)textureTransform;
  addLink(this, textureTransform_);
}

void Appearance::setLineProperties(const LineProperties *lineProperties)
{
  removeLink(this, lineProperties_);
  lineProperties_ = (SFNode)lineProperties;
  addLink(this, lineProperties_);
}

void Appearance::setFillProperties(const FillProperties *fillProperties)
{
  removeLink(this, fillProperties_);
  fillProperties_ = (SFNode)fillProperties;
  addLink(this, fillProperties_);
}

bool Appearance::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    if (material_ != 0)
      return false;
      
    setMaterial(static_cast<X3DMaterialNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    if (texture_ != 0)
      return false;
      
    setTexture(static_cast<X3DTextureNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    if (textureTransform_ != 0)
      return false;
      
    setTextureTransform(static_cast<X3DTextureTransformNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    if (lineProperties_ != 0)
      return false;
      
    setLineProperties(static_cast<LineProperties *>(N));
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    if (fillProperties_ != 0)
      return false;
      
    setFillProperties(static_cast<FillProperties *>(N));
    return true;    
  }
  
  return false;
}

bool Appearance::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    setMaterial(static_cast<X3DMaterialNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    setTexture(static_cast<X3DTextureNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    setTextureTransform(static_cast<X3DTextureTransformNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    setLineProperties(static_cast<LineProperties *>(N));
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    setFillProperties(static_cast<FillProperties *>(N));
    return true;    
  }
  
  return false;
}

bool Appearance::removeChild(const SFNode &N)
{
  if (material_ == N)
  {
    setMaterial(0);
    return true;
  }
  
  if (texture_ == N)
  {
    setTexture(0);
    return true; 
  }
  
  if (textureTransform_ == N)
  {
    setTextureTransform(0);    
    return true;  
  }
  
  if (lineProperties_ == N)
  {
    setLineProperties(0);
    return true;
  }
  
  if (fillProperties_ == N)
  {
    setFillProperties(0);
    return true; 
  }
  
  return false;
}
