/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Appearance.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef APPEARANCE_H
#define APPEARANCE_H

#include "X3DTypes.h"
#include "X3DAppearanceNode.h"

namespace X3DTK {

class X3DMaterialNode;
class X3DTextureNode;
class X3DTextureTransformNode;
class LineProperties;
class FillProperties;

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/shape.html">Shape 
/// component</a> defining an Appearance
/// (<a href="../../doc/X3D_Specs/Part01/components/Shape.html#Appearance">
/// X3D documentation</a>).

class Appearance : public X3DAppearanceNode
{
public:
  /// Constructor.
  Appearance();
  /// Constructs a Material from its attributes.
  Appearance(const X3DMaterialNode *material, 
             const X3DTextureNode *texture, 
	     const X3DTextureTransformNode *textureTransform, 
	     const LineProperties *lineProperties, 
	     const FillProperties *fillProperties);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Appearance();
  
  /// Sets the Material attribute.
  void setMaterial(const X3DMaterialNode *material);
  /// Sets the Texture attribute.
  void setTexture(const X3DTextureNode *texture);
  /// Sets the TextureTransform attribute.
  void setTextureTransform(const X3DTextureTransformNode *textureTransform);
  /// Sets the LineProperties attribute.
  void setLineProperties(const LineProperties *lineProperties);
  /// Sets the FillProperties attribute.
  void setFillProperties(const FillProperties *fillProperties);

  /// Gets the Material attribute.  
  inline SFNode getMaterial() const {return material_;};
  /// Gets the Texture attribute.  
  inline SFNode getTexture() const {return texture_;};
  /// Gets the TextureTransform attribute.  
  inline SFNode getTextureTransform() const {return textureTransform_;};
  /// Gets the LineProperties attribute.  
  inline SFNode getLineProperties() const {return lineProperties_;};
  /// Gets the FillProperties attribute.  
  inline SFNode getFillProperties() const {return fillProperties_;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFNode &N);

protected:  
  /// Copy constructor. 	     
  Appearance(const Appearance &A);
  
private:
  SFNode material_;
  SFNode texture_;
  SFNode textureTransform_;
  SFNode lineProperties_;
  SFNode fillProperties_;
};

}

#endif
