#include "FillProperties.h"

using namespace X3DTK;

FillProperties::FillProperties()
: X3DAppearanceChildNode(), fillStyle_(SFString("")), hatchStyle_(1), hatchColor_(SFColor(1.0f, 1.0f, 1.0f))
{
  defineTagName("FillProperties", "Shape");
}

FillProperties::FillProperties(SFString fillStyle, SFInt32 hatchStyle, SFColor hatchColor)
: X3DAppearanceChildNode(), fillStyle_(fillStyle), hatchStyle_(hatchStyle), hatchColor_(hatchColor)
{
  defineTagName("FillProperties", "Shape");
}

FillProperties::FillProperties(const FillProperties &F)
: X3DAppearanceChildNode(F), fillStyle_(F.fillStyle_), hatchStyle_(F.hatchStyle_), hatchColor_(F.hatchColor_)
{
  defineTagName("FillProperties", "Shape");
}

SFNode FillProperties::clone() const
{
  return new FillProperties(*this);
}

FillProperties::~FillProperties()
{
}

void FillProperties::setFillStyle(SFString fillStyle)
{
  fillStyle_ = fillStyle;
}

void FillProperties::setHatchStyle(SFInt32 hatchStyle)
{
  hatchStyle_ = hatchStyle;
}

void FillProperties::setHatchColor(SFColor hatchColor)
{
  hatchColor_ = hatchColor;
}

void FillProperties::loadAttributes(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("fillStyle");
  if (index != -1)
    fillStyle_ = element->getAttribute(index);
  
  index = element->getIndexAttribute("hatchStyle");
  if (index != -1)
    hatchStyle_ = element->getAttribute(index).toInt();  
  
  index = element->getIndexAttribute("hatchColor");
  if (index != -1)
    hatchColor_ = SFColor(element->getAttribute(index));    
}

SFString FillProperties::writeAttributes() const
{
  SFString attr;
  if (fillStyle_ != "NONE")
    attr += " fillStyle = \"" + fillStyle_ + "\"";
  if (hatchStyle_ != 1)
    attr += " hatchStyle = \"" + toSFString(hatchStyle_) + "\"";  
  if (hatchColor_ != SFColor(1.0f, 1.0f, 1.0f))
    attr += " hatchColor = \"" + toSFString(hatchColor_) + "\"";
      
  return attr;
}
