/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Shape.h                                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef SHAPE_H
#define SHAPE_H

#include "X3DTypes.h"
#include "X3DShapeNode.h"
#include "X3DBoundedObject.h"

namespace X3DTK {

/// \addtogroup Shape
/// @{

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/shape.html">Shape 
/// component</a> defining a Shape
/// (<a href="../../doc/X3D_Specs/Part01/components/Shape.html#Shape">
/// X3D documentation</a>).

class Shape : public X3DShapeNode, public X3DBoundedObject
{
public:
  /// Constructor.
  Shape();
  /// Constructs a Shape from its attributes.
  Shape(const X3DGeometryNode *geometry, 
        const X3DAppearanceNode *appearance, 
        const SFVec3f &bboxCenter, 
        const SFVec3f &bboxSize);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Shape();
  
  /// loads the attributes of the node.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor. 	
  Shape(const Shape &S);
};

}

#endif
