/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            X3DTextureCoordinateNode.h                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DTEXTURECOORDINATENODE_H
#define X3DTEXTURECOORDINATENODE_H

#include "X3DTypes.h"
#include "X3DGeometricPropertyNode.h"

namespace X3DTK {

/// Base class of the <a href="../../doc/X3D_Specs/Part01/components/texturing.html">
/// Texturing component</a> defining an X3DTextureCoordinateNode
/// (<a href="../../doc/X3D_Specs/Part01/components/texturing.html#X3DTextureCoordinateNode">
/// X3D documentation</a>).

class X3DTextureCoordinateNode : public X3DGeometricPropertyNode
{
public:  
  /// Constructor.
  X3DTextureCoordinateNode();
  /// Constructs an X3DTextureCoordinateNode from its attributes.
  X3DTextureCoordinateNode(const MFVec2f &point);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DTextureCoordinateNode();

  /// Sets the Point attribute.
  void setPoint(const MFVec2f &point);
    
  /// Gets the Point attribute.
  inline MFVec2f &getPoint() const {return (MFVec2f &)point;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.	
  X3DTextureCoordinateNode(const X3DTextureCoordinateNode &N);  
  
  /// The point array.
  MFVec2f point;  
};

}

#endif
