/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            SFPoint.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef POINT3F_H
#define POINT3F_H

#include "SFVec3f.h"

#include <math.h>
#include <iostream>

namespace X3DTK {

class SFString;

/// class providing an implementation of a 3D float point. 
/// SFPoint3f is a point in an affin space, which different from a vector represented
/// by the SFVec3f class 

class SFPoint3f
{
public:
   /// x coordinates
  float x;
  /// y coordinates
  float y;
  /// z coordinates
  float z;

  /// default constructor
  SFPoint3f();
  /// constructor with x, y and z coordinates in parameters
  SFPoint3f(float x, float y, float z);
  /// copy constructor
  SFPoint3f(const SFPoint3f &p);
  /// overloaded assignment operator
  SFPoint3f &operator= (SFPoint3f v);
  /// explicit constructor from a string of which format is "x y z".
  explicit SFPoint3f(const SFString &s);
  // conversion from vector to point
  explicit SFPoint3f(const SFVec3f &V);

   /// overloaded operator+ 
  friend SFPoint3f operator+ (const SFPoint3f &v1, const SFVec3f &v2);
  /// overloaded operator- 
  friend SFPoint3f operator- (const SFPoint3f &v1, const SFVec3f &v2);
  /// overloaded operator- 
  friend SFVec3f operator- (const SFPoint3f &v1, const SFPoint3f &v2);
  /// overloaded operator* 
  friend SFPoint3f operator* (const float a, const SFPoint3f &v);
  /// norm of the point
  friend float distance(const SFPoint3f &A, const SFPoint3f &B);
};

}

#endif
