/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            X3DBaseTypes.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DBASETYPES_H
#define X3DBASETYPES_H

#include "SFString.h"
#include "SFVec3f.h"
#include "SFPoint3f.h"
#include "SFMatrix34f.h"

#include <iostream>
#include <string>
#include <list>
#include <vector>
#include <map>

#if defined(__GNUC__) && (__GNUC__ < 3)

#include <hash_map>
using __STD::hash_map;
using __STD::hash_multimap;
using __STD::hash;

#else

#include <ext/hash_map>
using __gnu_cxx::hash_map;
using __gnu_cxx::hash_multimap;
using __gnu_cxx::hash;

#endif


namespace X3DTK {

class X3DNode;
class X3DAbstractNode;

/// MFString class providing multiple SFString values 
class MFString : public std::vector<SFString>
{
public:
  MFString(){};
  explicit MFString(const SFString &s);
};

/// Bool class
typedef bool SFBool;

/// MFBool class providing multiple SFBool values 
class MFBool : public std::vector<SFBool>
{
public:
  MFBool(){};
  explicit MFBool(const SFString &s);
};

/// SFInt32 class
typedef int SFInt32;

/// MFInt32 class providing multiple SFInt32 values   
class MFInt32 : public std::vector<SFInt32>
{
public:
  MFInt32(){}; 
  MFInt32(size_type n, const SFInt32 &V = SFInt32());
  explicit MFInt32(const SFString &s);
};

/// Float class
typedef float SFFloat;

/// MFFloat class providing multiple SFFloat values   
class MFFloat : public std::vector<SFFloat>
{
public:
  MFFloat(){};
  MFFloat(size_type n, const SFFloat &V = SFFloat());
  explicit MFFloat(const SFString &s);
};

/// SFDouble class
typedef double SFDouble;

/// MFDouble class providing multiple SFDouble values   
class MFDouble : public std::vector<SFDouble>
{
public:
  MFDouble(){};
  explicit MFDouble(const SFString &s);
};

/// SFColorRGBA class
class SFColorRGBA
{
public:
  SFFloat r, g, b, a;
  SFColorRGBA(){};
  SFColorRGBA(SFFloat r, SFFloat g, SFFloat b, SFFloat a);
  explicit SFColorRGBA(const SFString &s);
  
  friend bool operator== (const SFColorRGBA &r1, const SFColorRGBA &r2);
  friend bool operator!= (const SFColorRGBA &r1, const SFColorRGBA &r2);
};

/// MFColorRGBA class providing multiple SFColorRGBA values   
class MFColorRGBA : public std::vector<SFColorRGBA>
{
public:
  MFColorRGBA(){};
  MFColorRGBA(size_type n, const SFColorRGBA &C = SFColorRGBA());
  explicit MFColorRGBA(const SFString &s);
};

/// SFColor class
class SFColor
{
public:
  SFFloat r, g, b;
  SFColor(){};
  SFColor(SFFloat r, SFFloat g, SFFloat b);
  explicit SFColor(const SFString &s);
  operator const SFColorRGBA() const;

  friend bool operator== (const SFColor &r1, const SFColor &r2);
  friend bool operator!= (const SFColor &r1, const SFColor &r2);
};


/// MFColor class providing multiple SFColor values   
class MFColor : public std::vector<SFColor>
{
public:
  MFColor(){};
  explicit MFColor(const SFString &s);
  operator const MFColorRGBA() const;
};

/// SFNode class 
typedef X3DAbstractNode *SFNode;
typedef std::list<SFNode> MFNode;
typedef std::pair<SFNode, SFNode> SFNodeLink;
typedef std::list<SFNodeLink> MFDNode;

/// SFVec2d class  
class SFVec2d
{
public:
  SFDouble x, y;
  SFVec2d();
  SFVec2d(SFDouble x, SFDouble y);
  explicit SFVec2d(const SFString &s);
};
  
/// MFVec2d class providing multiple SFVec2d values     
class MFVec2d : public std::vector<SFVec2d>
{
public:
  MFVec2d(){};
  explicit MFVec2d(const SFString &s);
};
  
/// SFVec2f class 
class SFVec2f
{
public:
  SFFloat x, y;
  SFVec2f();
  SFVec2f(SFFloat x, SFFloat y);
  explicit SFVec2f(const SFString &s);
};

/// MFVec2f class providing multiple SFVec2f values     
class MFVec2f : public std::vector<SFVec2f>
{
public:
  MFVec2f(){};
  explicit MFVec2f(const SFString &s);
};

/// SFVec3d class
class SFVec3d
{
public:
  SFDouble x, y, z;
  SFVec3d();
  SFVec3d(SFDouble x, SFDouble y, SFDouble z);
  explicit SFVec3d(const SFString &s);    
};

/// MFVec3d class providing multiple SFVec3d values     
class MFVec3d : public std::vector<SFVec3d>
{
public:
  MFVec3d(){};
  explicit MFVec3d(const SFString &s);
};

/// MFVec3f class providing multiple SFVec3f values   
class MFVec3f : public std::vector<SFVec3f>
{
public:
  MFVec3f(){};
  MFVec3f(size_type n, const SFVec3f &V = SFVec3f());
  explicit MFVec3f(const SFString &s);
};

/// SFRotation 
class SFRotation
{
public:
  SFFloat x, y, z, angle;
  SFRotation(){};
  SFRotation(SFFloat x, SFFloat y, SFFloat z, SFFloat angle);
  explicit SFRotation(const SFString &s);
  
  friend bool operator== (const SFRotation &r1, const SFRotation &r2);
  friend bool operator!= (const SFRotation &r1, const SFRotation &r2);
};


/// MFRotation class providing multiple SFRotation values     
class MFRotation : public std::vector<SFRotation>
{
public:
  MFRotation(){};
  explicit MFRotation(const SFString &s);
};


//constants
static const double PI = 3.14159265358979323846;
static const double radCoef = 180.0/PI;

/// function converting radians to degrees, used for GL rotations
inline SFFloat radiansToDegrees(SFFloat alpha)
{
  return radCoef*alpha;
}

struct eqstr
{
  inline bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) == 0;
  }
};

struct eqSFString
{
  inline bool operator()(const SFString &s1, const SFString &s2) const
  {
    return s1 == s2;
  }
};

}

#if defined(__GNUC__) && (__GNUC__ < 3)
__STL_BEGIN_NAMESPACE

  template<> struct hash<X3DTK::SFString>
  {
    size_t operator()(const X3DTK::SFString &x) const
    {
      return hash<const char*>()(x.c_str());
    }
  };

__STL_END_NAMESPACE

#else
namespace __gnu_cxx
{
  template<> struct hash<X3DTK::SFString>
  {
    size_t operator()(const X3DTK::SFString &x) const
    {
      return hash<const char*>()(x.c_str());
    }
  };
}
#endif


namespace X3DTK {

// other types
class Type;
class Component;
class CreationFunction;
class WalkingFunction;
class VisitingFunctions;
class EnterFunction;
class WalkOnFunction;
class LeaveFunction;

class X3DComponentCreator;
typedef SFNode (X3DComponentCreator::* ptrToCreationFunction)() const;

class X3DComponentWalker;
typedef void (X3DComponentWalker::* ptrToWalkingFunction)(SFNode) const;

class X3DComponentVisitor;
typedef void (X3DComponentVisitor::* ptrToEnterFunction)(SFNode) const;
typedef bool (X3DComponentVisitor::* ptrToWalkOnFunction)(SFNode, SFNode) const;
typedef void (X3DComponentVisitor::* ptrToLeaveFunction)(SFNode) const;

typedef hash_map<SFString, Type *, hash<SFString>, eqSFString> MFType;
typedef hash_map<SFString, Component *, hash<SFString>, eqSFString> MFComponent;
typedef hash_multimap<SFString, SFNode, hash<SFString>, eqSFString> X3DDict;

typedef hash_map<SFString, CreationFunction *, hash<SFString>, eqSFString> CreationDict;
typedef hash_map<SFString, WalkingFunction *, hash<SFString>, eqSFString> WalkingDict;
typedef hash_map<SFString, EnterFunction *, hash<SFString>, eqSFString> EnterDict;
typedef hash_map<SFString, WalkOnFunction *, hash<SFString>, eqSFString> WalkOnDict;
typedef hash_map<SFString, LeaveFunction *, hash<SFString>, eqSFString> LeaveDict;

typedef std::vector<WalkingFunction *> WalkingArray;
typedef std::vector<VisitingFunctions *> VisitingArray;

class X3DNodeProxy;
typedef std::list<X3DNodeProxy *> MFNodeProxy; 

/// operator<< overloaded for MFString class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFString &ms);
/// operator<< overloaded for MFBool class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFBool &mb);
/// operator<< overloaded for MFint32 class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFInt32 &mi);
/// operator<< overloaded for MFFloat class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFFloat &mf);
/// operator<< overloaded for MFDouble class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFDouble &md);
/// operator<< overloaded for SFColor class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFColor &c);
/// operator<< overloaded for MFColor class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFColor &mc);
/// operator<< overloaded for SFColorRGBA class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFColorRGBA &c);
/// operator<< overloaded for MFColorRGBA class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFColorRGBA &mc);
/// operator<< overloaded for SFVec2d class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFVec2d &v);
/// operator<< overloaded for MFVec2d class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFVec2d &mv);
/// operator<< overloaded for SFVec2f class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFVec2f &v);
/// operator<< overloaded for MFVec2f class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFVec2f &mv);
/// operator<< overloaded for SFVec3d class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFVec3d &v);
/// operator<< overloaded for MFVec3d class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFVec3d &mv);
/// operator<< overloaded for SFVec3f class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFVec3f &v);
/// operator<< overloaded for MFVec3f class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFVec3f &mv);
/// operator<< overloaded for SFPoint3f class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFPoint3f &v);
/// operator<< overloaded for SFRotation class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::SFRotation &r);
/// operator<< overloaded for MFRotation class respecting the X3D notation
std::ostream& operator<<(std::ostream& o, const X3DTK::MFRotation &mr);

}

#endif
