#include "X3DTypes.h"

#include <sstream>

using namespace X3DTK;
using namespace std;

MFString::MFString(const SFString &s)
{
  string token;
  string::const_iterator itChar = s.begin();
  size_t counter = 0;
  size_t begin = 0;
  size_t size = 0;
  
  while (itChar != s.end())
  {
    if (*itChar == '\"')
    {
      ++itChar;
      ++counter;
      begin = counter;
      size = 0;
      
      while ((*itChar != '\"') && (itChar != s.end()))
      {    
        ++itChar;
        ++counter;
        ++size;
      }
      token = s.substr(begin, size);
      this->push_back(token);
    }
    ++itChar;
    ++counter;
  }
}

MFBool::MFBool(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  bool b;
  while (!iss.eof())
  {
    iss >> b;
    this->push_back(b);
  }  
}

MFInt32::MFInt32(size_type n, const SFInt32 &V)
: vector<SFInt32>(n, V)
{
}

MFInt32::MFInt32(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  SFInt32 i;
  while (!iss.eof())
  {
    iss >> i;
    this->push_back(i);
  }  
}

MFFloat::MFFloat(size_type n, const SFFloat &V)
: vector<SFFloat>(n, V)
{
}

MFFloat::MFFloat(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  SFFloat f;
  while (!iss.eof())
  {
    iss >> f;
    this->push_back(f);
  }  
}

MFDouble::MFDouble(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  SFDouble d;
  while (!iss.eof())
  {
    iss >> d;
    this->push_back(d);
  }  
}

SFColorRGBA::SFColorRGBA(SFFloat r, SFFloat g, SFFloat b, SFFloat a)
{
  this->r = r;
  this->g = g;
  this->b = b;
  this->a = a;
}

bool X3DTK::operator== (const SFColorRGBA &r1, const SFColorRGBA &r2)
{
  return ((r1.r == r2.r) && (r1.g == r2.g) && (r1.b == r2.b) && (r1.a == r2.a));
}

bool X3DTK::operator!= (const SFColorRGBA &r1, const SFColorRGBA &r2)
{
  return ((r1.r != r2.r) || (r1.g != r2.g) || (r1.b != r2.b) || (r1.a != r2.a));
}


SFColorRGBA::SFColorRGBA(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  iss >> r >> g >> b >> a;
}

SFString toSFString(const SFColorRGBA c)
{
  return SFString::number(c.r) + SFString(" ") 
       + SFString::number(c.g) + SFString(" ")
       + SFString::number(c.b) + SFString(" ")
       + SFString::number(c.a);
}

MFColorRGBA::MFColorRGBA(size_type n, const SFColorRGBA &C)
: vector<SFColorRGBA>(n, C)
{
}

MFColorRGBA::MFColorRGBA(const SFString &s)
{
  SFString copy(s);
  replace(copy.begin(), copy.end(), ',', ' ');
  istringstream iss(copy, istringstream::in);
  while (!iss.eof())
  {
    SFColorRGBA c;
    iss >> c.r >> c.g >> c.b >> c.a;
    this->push_back(c);
  }  
}

SFColor::SFColor(SFFloat r, SFFloat g, SFFloat b)
{
  this->r = r;
  this->g = g;
  this->b = b;
};

bool X3DTK::operator== (const SFColor &r1, const SFColor &r2)
{
  return ((r1.r == r2.r) && (r1.g == r2.g) && (r1.b == r2.b));
}

bool X3DTK::operator!= (const SFColor &r1, const SFColor &r2)
{
  return ((r1.r != r2.r) || (r1.g != r2.g) || (r1.b != r2.b));
}

SFColor::SFColor(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  iss >> r >> g >> b;
}

SFColor::operator const SFColorRGBA() const
{
  return SFColorRGBA(r, g, b, 1.0f);
}

MFColor::MFColor(const SFString &s)
{
  SFString copy(s);
  replace(copy.begin(), copy.end(), ',', ' ');
  istringstream iss(copy, istringstream::in);
  while (!iss.eof())
  {
    SFColor c;
    iss >> c.r >> c.g >> c.b;
    this->push_back(c);
  }  
}

MFColor::operator const MFColorRGBA() const
{
  MFColorRGBA color = MFColorRGBA(this->size());
  MFColorRGBA::iterator itColor = color.begin();
  for (const_iterator it = this->begin(); it != this->end(); ++it)
  {
    *itColor = *it;
    ++itColor;
  }  

  return color;
}

SFVec2d::SFVec2d()
{
}

SFVec2d::SFVec2d(SFDouble x, SFDouble y)
{
  this->x = x;
  this->y = y;
}

SFVec2d::SFVec2d(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  iss >> x >> y;
}

MFVec2d::MFVec2d(const SFString &s)
{
  SFString copy(s);
  replace(copy.begin(), copy.end(), ',', ' ');
  istringstream iss(copy, istringstream::in);
  while (!iss.eof())
  {
    SFVec2d v;
    iss >> v.x >> v.y;
    this->push_back(v);
  }  
}

SFVec2f::SFVec2f()
{
}

SFVec2f::SFVec2f(SFFloat x, SFFloat y)
{
  this->x = x;
  this->y = y;
}

SFVec2f::SFVec2f(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  iss >> x >> y;
}

MFVec2f::MFVec2f(const SFString &s)
{
  SFString copy(s);
  replace(copy.begin(), copy.end(), ',', ' ');
  istringstream iss(copy, istringstream::in);
  while (!iss.eof())
  {
    SFVec2f v;
    iss >> v.x >> v.y;
    this->push_back(v);
  }  
}

SFVec3d::SFVec3d()
{
}


SFVec3d::SFVec3d(SFDouble x, SFDouble y, SFDouble z)
{
  this->x = x;
  this->y = y;
  this->z = z;
}

SFVec3d::SFVec3d(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  iss >> x >> y >> z;
}

MFVec3d::MFVec3d(const SFString &s)
{
  SFString copy(s);
  replace(copy.begin(), copy.end(), ',', ' ');
  istringstream iss(copy, istringstream::in);
  while (!iss.eof())
  {
    SFVec3d v;
    iss >> v.x >> v.y >> v.z;
    this->push_back(v);
  }  
}

MFVec3f::MFVec3f(size_type n, const SFVec3f &V)
: vector<SFVec3f>(n, V)
{
}

MFVec3f::MFVec3f(const SFString &s)
: vector<SFVec3f>()
{
  SFString copy(s);
  replace(copy.begin(), copy.end(), ',', ' ');
  istringstream iss(copy, istringstream::in);
  while (!iss.eof())
  {
    SFVec3f v;
    iss >> v.x >> v.y >> v.z;
    this->push_back(v);
  }  
}

SFRotation::SFRotation(SFFloat x, SFFloat y, SFFloat z, SFFloat angle)
{
  this->x = x;
  this->y = y;
  this->z = z;
  this->angle = angle;
}

SFRotation::SFRotation(const SFString &s)
{
  istringstream iss(s, istringstream::in);
  iss >> x >> y >> z >> angle;
}

bool X3DTK::operator== (const SFRotation &r1, const SFRotation &r2)
{
  return ((r1.x == r2.x) && (r1.y == r2.y) && (r1.z == r2.z));
}

bool X3DTK::operator!= (const SFRotation &r1, const SFRotation &r2)
{
  return ((r1.x != r2.x) || (r1.y != r2.y) || (r1.z != r2.z));
}

MFRotation::MFRotation(const SFString &s)
{
  SFString copy(s);
  replace(copy.begin(), copy.end(), ',', ' ');
  istringstream iss(copy, istringstream::in);
  while (!iss.eof())
  {
    SFRotation r;
    iss >> r.x >> r.y >> r.z >> r.angle;
    this->push_back(r);
  }  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFString &ms)
{
if (!ms.empty())
  {
    for (unsigned int i = 0; i < ms.size() - 1; ++i)
      o << ms[i] << " ";
  
    o << ms.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFBool &mb)
{
  for (unsigned int i = 0; i < mb.size() - 1; ++i)
  {
    if (mb[i])
      o << "TRUE, ";
    else
      o << "FALSE, ";        
  }  
  
  if (!mb.empty())
  {
    if (mb.back())
      o << "TRUE";
    else
      o << "FALSE";        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFInt32 &mi)
{
  if (!mi.empty())
  {
    for (unsigned int i = 0; i < mi.size() - 1; ++i)
      o << mi[i] << " ";
  
    o << mi.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFFloat &mf)
{
  if (!mf.empty())
  {
    for (unsigned int i = 0; i < mf.size() - 1; ++i)
      o << mf[i] << " ";
    
    o << mf.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFDouble &md)
{
  if (!md.empty())
  {
    for (unsigned int i = 0; i < md.size() - 1; ++i)
      o << md[i] << " ";

    o << md.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFColor &c)
{
  return o << c.r << " " << c.g << " " << c.b;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFColor &mc)
{
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
      o << mc[i] << ", ";
    
    o << mc.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFColorRGBA &c)
{
  return o << c.r << " " << c.g << " " << c.b << " " << c.a;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFColorRGBA &mc)
{
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
      o << mc[i] << ", ";
    
    o << mc.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec2d &v)
{
  return o << v.x << " " << v.y ;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec2d &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
    
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec2f &v)
{
  return o << v.x << " " << v.y;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec2f &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
  
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec3d &v)
{
  return o << v.x << " " << v.y << " " << v.z;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec3d &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
  
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec3f &v)
{
  return o << v.x << " " << v.y << " " << v.z;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec3f &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
  
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFPoint3f &v)
{
  return o << v.x << " " << v.y << " " << v.z;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFRotation &r)
{
  return o << r.x << " " << r.y << " " << r.z << " " << r.angle;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFRotation &mr)
{
  if (!mr.empty())
  {
    for (unsigned int i = 0; i < mr.size() - 1; ++i)
      o << mr[i] << ", ";
  
    o << mr.back();        
  }  
  return o;  
}
