#include "BboxUpdaterGeometry3DVisitor.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "Sphere.h"
#include "Bbox.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

BboxUpdaterGeometry3DVisitor::BboxUpdaterGeometry3DVisitor()
: Geometry3DVisitor()
{
  // Enter functions.
  defineNewEnterFunction<BboxUpdaterGeometry3DVisitor, Box>(&BboxUpdaterGeometry3DVisitor::enterBox);
  defineNewEnterFunction<BboxUpdaterGeometry3DVisitor, Cone>(&BboxUpdaterGeometry3DVisitor::enterCone);
  defineNewEnterFunction<BboxUpdaterGeometry3DVisitor, Cylinder>(&BboxUpdaterGeometry3DVisitor::enterCylinder);
  defineNewEnterFunction<BboxUpdaterGeometry3DVisitor, Sphere>(&BboxUpdaterGeometry3DVisitor::enterSphere);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<BboxUpdaterGlobalVariables>();
}

BboxUpdaterGeometry3DVisitor::~BboxUpdaterGeometry3DVisitor()
{
}

void BboxUpdaterGeometry3DVisitor::enterBox(Box *B) const
{
  Bbox *BB = globalVariables->getBbox(B);
  if (BB == 0)
  {
    BB = new Bbox(SFVec3f(0.0f, 0.0f, 0.0f), B->getSize());
    globalVariables->addBbox(B, BB);  
  }
  globalVariables->setCurrentBbox(BB);  
}

void BboxUpdaterGeometry3DVisitor::enterCone(Cone *C) const
{
  Bbox *BB = globalVariables->getBbox(C);
  if (BB == 0)
  {
    BB = new Bbox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(2.0f*C->getBottomRadius(), C->getHeight(), 2.0f*C->getBottomRadius()));
    globalVariables->addBbox(C, BB);  
  }
  globalVariables->setCurrentBbox(BB);
}

void BboxUpdaterGeometry3DVisitor::enterCylinder(Cylinder *C) const
{
  Bbox *BB = globalVariables->getBbox(C);
  if (BB == 0)
  {
    BB = new Bbox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(2.0f*C->getRadius(), C->getHeight(), 2.0f*C->getRadius()));
    globalVariables->addBbox(C, BB);  
  }
  globalVariables->setCurrentBbox(BB);
}

void BboxUpdaterGeometry3DVisitor::enterSphere(Sphere *S) const
{
  Bbox *BB = globalVariables->getBbox(S);
  if (BB == 0)
  {
    BB = new Bbox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(2.0f*S->getRadius(), 2.0f*S->getRadius(), 2.0f*S->getRadius()));
    globalVariables->addBbox(S, BB);  
  }
  globalVariables->setCurrentBbox(BB);
}

