#include "BboxUpdaterRenderingVisitor.h"
#include "Coordinate.h"
#include "X3DGeometryNode.h"

using namespace X3DTK;

BboxUpdaterRenderingVisitor::BboxUpdaterRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<BboxUpdaterRenderingVisitor, Coordinate>(&BboxUpdaterRenderingVisitor::enterCoordinate);

  // WalkOn functions.
  defineNewWalkOnFunction<BboxUpdaterRenderingVisitor, X3DGeometryNode>(&BboxUpdaterRenderingVisitor::walkOnX3DGeometryNode);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<BboxUpdaterGlobalVariables>();
}

BboxUpdaterRenderingVisitor::~BboxUpdaterRenderingVisitor()
{
}

void BboxUpdaterRenderingVisitor::enterCoordinate(Coordinate *C) const
{
  Bbox *BB = globalVariables->getBbox(C);
  if (BB == 0)
  {
    MFVec3f &vertexArray = C->getPoint();
  
    if (vertexArray.empty())
    {
      BB = new Bbox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(-1.0f, -1.0f, -1.0f));
      return;
    }
  
    SFVec3f min = vertexArray[0];
    SFVec3f max = vertexArray[0];
  
    for (MFVec3f::const_iterator itVertex = vertexArray.begin(); itVertex != vertexArray.end(); ++itVertex)
    {
      if ((*itVertex).x < min.x)
        min.x = (*itVertex).x;
      if ((*itVertex).y < min.y)
        min.y = (*itVertex).y;
      if ((*itVertex).z < min.z)
        min.z = (*itVertex).z;
   
      if ((*itVertex).x > max.x)
        max.x = (*itVertex).x;
      if ((*itVertex).y > max.y)
        max.y = (*itVertex).y;
      if ((*itVertex).z > max.z)
        max.z = (*itVertex).z;
    }
  
    //computing the new center
    SFVec3f center = 0.5f*(min + max);
    BB = new Bbox(center, 2.0f*(max - center));  
    globalVariables->addBbox(C, BB);  
  }
  globalVariables->setCurrentBbox(BB);
}

bool BboxUpdaterRenderingVisitor::walkOnX3DGeometryNode(X3DGeometryNode *N, SFNode Child) const
{
  return true;
}

