#include "GLRendererGLGeometry3DVisitor.h"
#include "GLBox.h"
#include "GLCone.h"
#include "GLCylinder.h"
#include "GLIndexedFaceSet.h"
#include "GLSphere.h"
#include "BoxDrawArray.h"
#include "ConeDrawArray.h"
#include "CylinderDrawArray.h"
#include "SphereDrawArray.h"

using namespace X3DTK;

GLRendererGLGeometry3DVisitor::GLRendererGLGeometry3DVisitor()
: GLGeometry3DVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLRendererGLGeometry3DVisitor, GLBox>(&GLRendererGLGeometry3DVisitor::enterGLBox);
  defineNewEnterFunction<GLRendererGLGeometry3DVisitor, GLCone>(&GLRendererGLGeometry3DVisitor::enterGLCone);
  defineNewEnterFunction<GLRendererGLGeometry3DVisitor, GLCylinder>(&GLRendererGLGeometry3DVisitor::enterGLCylinder);
  defineNewEnterFunction<GLRendererGLGeometry3DVisitor, GLIndexedFaceSet>(&GLRendererGLGeometry3DVisitor::enterGLIndexedFaceSet);
  defineNewEnterFunction<GLRendererGLGeometry3DVisitor, GLSphere>(&GLRendererGLGeometry3DVisitor::enterGLSphere);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLRendererGlobalVariables>();
}

GLRendererGLGeometry3DVisitor::~GLRendererGLGeometry3DVisitor()
{
}

void GLRendererGLGeometry3DVisitor::enterGLBox(GLBox *G) const
{
  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CW);
  glCullFace(GL_BACK);    
  glPushMatrix();
  
  glScalef(G->getSize().x, G->getSize().y, G->getSize().z);  
  
  glInterleavedArrays(GL_N3F_V3F, 0, G->getBoxDrawArray()->getBoxVertexArrayAddress());
  glDrawElements(GL_TRIANGLES, G->getBoxDrawArray()->getBoxSize(), GL_UNSIGNED_INT, G->getBoxDrawArray()->getBoxIndexArrayAddress());

  glPopMatrix();
  glDisable(GL_CULL_FACE);
}

void GLRendererGLGeometry3DVisitor::enterGLCone(GLCone *G) const
{
  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CW);
  glCullFace(GL_BACK);    
  glPushMatrix();

  glScalef(G->getBottomRadius(), G->getHeight(), G->getBottomRadius());  
  
  //test the attributes
  if (G->getSide())
  {
    glInterleavedArrays(GL_N3F_V3F, 0, G->getConeDrawArray()->getConeSideVertexArrayAddress());
    glDrawElements(GL_TRIANGLE_FAN, G->getConeDrawArray()->getConeSideSize(), GL_UNSIGNED_INT, G->getConeDrawArray()->getConeSideIndexArrayAddress());
  }  
  if (G->getBottom())
  {
    glInterleavedArrays(GL_N3F_V3F, 0, G->getConeDrawArray()->getConeBottomVertexArrayAddress());
    glDrawElements(GL_TRIANGLE_FAN, G->getConeDrawArray()->getConeBottomSize(), GL_UNSIGNED_INT, G->getConeDrawArray()->getConeBottomIndexArrayAddress());
  }  
  
  glPopMatrix();
  glDisable(GL_CULL_FACE);
}

void GLRendererGLGeometry3DVisitor::enterGLCylinder(GLCylinder *G) const
{
  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CW);
  glCullFace(GL_BACK);    
  glPushMatrix();

  glScalef(G->getRadius(), G->getHeight(), G->getRadius());  
  
  //test the attributes
  if (G->getBottom())
  {
    glInterleavedArrays(GL_N3F_V3F, 0, G->getCylinderDrawArray()->getCylinderBottomVertexArrayAddress());
    glDrawElements(GL_TRIANGLE_FAN, G->getCylinderDrawArray()->getCylinderBottomSize(), GL_UNSIGNED_INT, G->getCylinderDrawArray()->getCylinderBottomIndexArrayAddress());
  } 
  if (G->getSide())
  {
    glInterleavedArrays(GL_N3F_V3F, 0, G->getCylinderDrawArray()->getCylinderSideVertexArrayAddress());
    glDrawElements(GL_TRIANGLE_STRIP, G->getCylinderDrawArray()->getCylinderSideSize(), GL_UNSIGNED_INT, G->getCylinderDrawArray()->getCylinderSideIndexArrayAddress());
  }  
  if (G->getTop())
  {
    glInterleavedArrays(GL_N3F_V3F, 0, G->getCylinderDrawArray()->getCylinderTopVertexArrayAddress());
    glDrawElements(GL_TRIANGLE_FAN, G->getCylinderDrawArray()->getCylinderTopSize(), GL_UNSIGNED_INT, G->getCylinderDrawArray()->getCylinderTopIndexArrayAddress());
  }  
  
  glPopMatrix();
  glDisable(GL_CULL_FACE);
}

void GLRendererGLGeometry3DVisitor::enterGLIndexedFaceSet(GLIndexedFaceSet *G) const
{
  if (G->getSolid())
  {
    glEnable(GL_CULL_FACE);
    if (G->getCcw())
      glFrontFace(GL_CCW);
    else
      glFrontFace(GL_CW);
    glCullFace(GL_BACK);   
  }
  
  if ((G->getColor()) && (G->getTexCoord()))
  {
  }  
    
  if ((G->getColor()) && (!G->getTexCoord()))
  {
    glEnable(GL_COLOR_MATERIAL);
    if (!G->getVerticesDuplicated())
    {
      glInterleavedArrays(GL_C4F_N3F_V3F, 0, &G->getC4F_N3F_V3FvertexFaceArray().front());
      glDrawElements(GL_TRIANGLES, G->getIndexFaceArray().size(), GL_UNSIGNED_INT, &G->getIndexFaceArray().front());
    }
    else
    {
      glInterleavedArrays(GL_C4F_N3F_V3F, 0, &G->getC4F_N3F_V3FvertexFaceArray().front());
      glDrawArrays(GL_TRIANGLES, 0, G->getC4F_N3F_V3FvertexFaceArray().size());
    }
    glDisable(GL_COLOR_MATERIAL);
  }  
    
  if ((!G->getColor()) && (G->getTexCoord()))
  {
    glEnable(GL_TEXTURE_2D);
    if (!G->getVerticesDuplicated())
    {
      glInterleavedArrays(GL_T2F_N3F_V3F, 0, &G->getT2F_N3F_V3FvertexFaceArray().front());
      glDrawElements(GL_TRIANGLES, G->getIndexFaceArray().size(), GL_UNSIGNED_INT, &G->getIndexFaceArray().front());
    }
    else
    {
      glInterleavedArrays(GL_T2F_N3F_V3F, 0, &(G->getT2F_N3F_V3FvertexFaceArray().front()));
      glDrawArrays(GL_TRIANGLES, 0, G->getT2F_N3F_V3FvertexFaceArray().size());
    }
    glDisable(GL_TEXTURE_2D);
  }  
  
  if ((!G->getColor()) && (!G->getTexCoord()))
  {
    if (!G->getVerticesDuplicated())
    {
      //drawing : not optimized because one call per polygon => too much if polygons
      //are triangles!
      glInterleavedArrays(GL_N3F_V3F, 0, &G->getN3F_V3FvertexFaceArray().front());
      glDrawElements(GL_TRIANGLES, G->getIndexFaceArray().size(), GL_UNSIGNED_INT, &G->getIndexFaceArray().front());
    }
    else
    {
      glInterleavedArrays(GL_N3F_V3F, 0, &G->getN3F_V3FvertexFaceArray().front());
      glDrawArrays(GL_TRIANGLES, 0, G->getN3F_V3FvertexFaceArray().size());
    }
  }
  
  if (G->getSolid())
    glDisable(GL_CULL_FACE);  
}

void GLRendererGLGeometry3DVisitor::enterGLSphere(GLSphere *G) const
{
  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CW);
  glCullFace(GL_BACK);    
  glPushMatrix();
  glScalef(G->getRadius(), G->getRadius(), G->getRadius());  

  glInterleavedArrays(GL_N3F_V3F, 0, G->getSphereDrawArray()->getSphereVertexArrayAddress());
  glDrawElements(GL_TRIANGLES, G->getSphereDrawArray()->getSphereSize(), GL_UNSIGNED_INT, G->getSphereDrawArray()->getSphereIndexArrayAddress());

  glPopMatrix();
  glDisable(GL_CULL_FACE);
}

