#include "GLUpdaterGLGeometry3DVisitor.h"
#include "GLBox.h"
#include "BoxDrawArray.h"
#include "GLCone.h"
#include "ConeDrawArray.h"
#include "GLCylinder.h"
#include "CylinderDrawArray.h"
#include "GLIndexedFaceSet.h"
#include "GLSphere.h"
#include "SphereDrawArray.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "IndexedFaceSet.h"
#include "Sphere.h"
#include "Normal.h"
#include "Coordinate.h"
#include "TextureCoordinate.h"
#include "Color.h"
#include "ColorRGBA.h"

#include <vector>

using namespace X3DTK;
using namespace std;

GLUpdaterGLGeometry3DVisitor::GLUpdaterGLGeometry3DVisitor()
: GLGeometry3DVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLBox>(&GLUpdaterGLGeometry3DVisitor::enterGLBox);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLCone>(&GLUpdaterGLGeometry3DVisitor::enterGLCone);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLCylinder>(&GLUpdaterGLGeometry3DVisitor::enterGLCylinder);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLIndexedFaceSet>(&GLUpdaterGLGeometry3DVisitor::enterGLIndexedFaceSet);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLSphere>(&GLUpdaterGLGeometry3DVisitor::enterGLSphere);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLUpdaterGlobalVariables>();
}

GLUpdaterGLGeometry3DVisitor::~GLUpdaterGLGeometry3DVisitor()
{
}

void GLUpdaterGLGeometry3DVisitor::enterGLBox(GLBox *G) const
{
  Box *B = static_cast<Box *>(G->getLink());

  G->getSize() = B->getSize();
  G->getBoxDrawArray() = BoxDrawArray::getInstance();
}

void GLUpdaterGLGeometry3DVisitor::enterGLCone(GLCone *G) const
{
  Cone *C = static_cast<Cone *>(G->getLink());

  G->getBottomRadius() = C->getBottomRadius();
  G->getHeight() = C->getHeight();
  G->getSide() = C->getSide();
  G->getBottom() = C->getBottom();
  G->getConeDrawArray() = ConeDrawArray::getInstanceOfSection(64);
}

void GLUpdaterGLGeometry3DVisitor::enterGLCylinder(GLCylinder *G) const
{
  Cylinder *C = static_cast<Cylinder *>(G->getLink());
  
  G->getBottom() = C->getBottom();
  G->getRadius() = C->getRadius();
  G->getHeight() = C->getHeight();
  G->getSide() = C->getSide();
  G->getTop() = C->getTop();
  G->getCylinderDrawArray() = CylinderDrawArray::getInstanceOfSection(64);
}

void GLUpdaterGLGeometry3DVisitor::enterGLSphere(GLSphere *G) const
{
  Sphere *S = static_cast<Sphere *>(G->getLink());

  G->getRadius() = S->getRadius();
  G->getSphereDrawArray() = SphereDrawArray::getInstanceOfSection(64);  
}

void GLUpdaterGLGeometry3DVisitor::enterGLIndexedFaceSet(GLIndexedFaceSet *G) const
{
  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());  

  //for the moment if the node color has changed, all the attributes are computed
  //emptying the arrays
  G->getN3F_V3FvertexFaceArray() = vector<N3F_V3F>();
  G->getC4F_N3F_V3FvertexFaceArray() = vector<C4F_N3F_V3F>(); 
  G->getT2F_N3F_V3FvertexFaceArray() = vector<T2F_N3F_V3F>();
  G->getT2F_C4F_N3F_V3FvertexFaceArray() = vector<T2F_C4F_N3F_V3F>(); 
  G->getIndexFaceArray() = vector<unsigned int>();

  MFVec3f vertexArray;
  MFVec3f normalArray;
  MFInt32 coordIndex;

  Normal *NormalNode = static_cast<Normal *>(I->getNormal());

  if (NormalNode != 0)
    normalArray = NormalNode->getVector();

  //IndexedFaceSet attributes  
  if (I->getCoordIndex().empty())
    return;
 
  G->getSolid() = I->getSolid();
  G->getCcw() = I->getCcw();

  computeArrays(G, vertexArray, normalArray, coordIndex); 

  if ((I->getColor() != 0) && (I->getTexCoord() != 0))
  {
    G->getColor() = true;
    G->getTexCoord() = true;
    computeColor_TexCoord(G, vertexArray, normalArray, coordIndex);
  }
  if ((I->getColor() != 0) && (I->getTexCoord() == 0))
  {
    G->getColor() = true;
    G->getTexCoord() = false;
    computeColor_NoTexCoord(G, vertexArray, normalArray, coordIndex);
  }
  if ((I->getColor() == 0) && (I->getTexCoord() != 0))
  {
    G->getColor() = false;
    G->getTexCoord() = true;
    computeNoColor_TexCoord(G, vertexArray, normalArray, coordIndex);
  }
  if ((I->getColor() == 0) && (I->getTexCoord() == 0))
  {
    G->getColor() = false;
    G->getTexCoord() = false;
    computeNoColor_NoTexCoord(G, vertexArray, normalArray, coordIndex);
  }
}

void GLUpdaterGLGeometry3DVisitor::computeNoColor_NoTexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: no color, no texCoord" << endl;

  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;

  SFBool normalPerVertex = I->getNormalPerVertex();
  MFInt32 colorIndex = I->getColorIndex(); 
  MFInt32 normalIndex = I->getNormalIndex(); 
  
  //using the GL_N3F_V3F
  //vertex array
  if ((normalPerVertex) 
   && (normalIndex.empty()))
  {
    //cout << "vertices not duplicated" << endl;
    
    G->getVerticesDuplicated() = false;
    //using the GL_POLYGON 
    //filling the vertex array
    MFVec3f::const_iterator itNormal = normalArray.begin();
    
    G->getN3F_V3FvertexFaceArray() = vector<N3F_V3F>(vertexArray.size());
    vector<N3F_V3F>::iterator itElement = G->getN3F_V3FvertexFaceArray().begin();
      
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      (*itElement).normal = *itNormal; 
      (*itElement).vertex = *itCoord;
      ++itNormal;
      ++itElement;
    }
      
    //filling the index array
    //iterate the coordIndex
    unsigned int faces = coordIndex.size()/4;
    G->getIndexFaceArray() = vector<unsigned int>(3*faces);//triangles are only considered
    vector<unsigned int>::iterator itIndex = G->getIndexFaceArray().begin();
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    
    for (unsigned int i = 0; i < faces; ++i)
    {
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      ++itCoordIndex;//-1
    }
  }
  else 
  {
    //cout << "vertices duplicated" << endl;
  
    G->getVerticesDuplicated() = true;
    //we cannot use the index array because an attribute is not constant per
    //instantiation of vertex
    //using the GL_POLYGON 
    //filling the arrays
    unsigned int faces = coordIndex.size()/4;
    G->getN3F_V3FvertexFaceArray() = vector<N3F_V3F>(3*faces);
    vector<N3F_V3F>::iterator itVertex = G->getN3F_V3FvertexFaceArray().begin();
    
    MFInt32::const_iterator itNormalIndex = normalIndex.begin();
    //iterate the coordIndex
    
    for (MFInt32::const_iterator itCoordIndex = coordIndex.begin(); itCoordIndex != coordIndex.end(); ++itCoordIndex)
    {
      if (*itCoordIndex == -1)
        ++itNormalIndex;
      else
      {
        if (normalIndex.empty())
          (*itVertex).normal = normalArray[*itCoordIndex];
        else
        {
          (*itVertex).normal = normalArray[*itNormalIndex];
          if (normalPerVertex) 
            ++itNormalIndex;
         }
        (*itVertex).vertex = vertexArray[*itCoordIndex];
        ++itVertex;
      }
    }
  }
}

void GLUpdaterGLGeometry3DVisitor::computeColor_NoTexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: color, no texCoord" << endl;

  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());

  MFColorRGBA colorRGBA;
  if (I->getColor()->getName() == "ColorRGBA")
    colorRGBA = static_cast<ColorRGBA *>(I->getColor())->getColor();
  else  
    colorRGBA = static_cast<Color *>(I->getColor())->getColor();
  
  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;
  
  SFBool normalPerVertex = I->getNormalPerVertex();
  SFBool colorPerVertex = I->getColorPerVertex();
  MFInt32 colorIndex = I->getColorIndex(); 
  MFInt32 normalIndex = I->getNormalIndex(); 
  
  //using the GL_C4F_N3F_V3F
  //vertex array
  if ((normalPerVertex)
   && (colorPerVertex) 
   && (colorIndex.empty()) 
   && (normalIndex.empty()))
  {
    //cout << "vertices not duplicated" << endl;

    G->getVerticesDuplicated() = false;
    //using the GL_POLYGON 
    //filling the vertex array
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
    
    G->getC4F_N3F_V3FvertexFaceArray() = vector<C4F_N3F_V3F>(vertexArray.size());
    vector<C4F_N3F_V3F>::iterator itElement = G->getC4F_N3F_V3FvertexFaceArray().begin();
        
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      (*itElement).color = *itColor;
      (*itElement).normal = *itNormal; 
      (*itElement).vertex = *itCoord;
      ++itColor;
      ++itNormal;
      ++itElement;
    }
    //filling the index array
    //iterate the coordIndex
    unsigned int faces = coordIndex.size()/4;
    G->getIndexFaceArray() = vector<unsigned int>(3*faces);//triangles are only considered
    vector<unsigned int>::iterator itIndex = G->getIndexFaceArray().begin();
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    
    for (unsigned int i = 0; i < faces; ++i)
    {
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      ++itCoordIndex;//-1
    }
  }
  else //normalPerVertex or colorPertex is FALSE or ...
  {

    //cout << "vertices duplicated" << endl;

    G->getVerticesDuplicated() = true;
    //we cannot use the index array because an attribute is not constant per
    //instantiation of vertex
    //using the GL_POLYGON 
    //filling the vertex array
    //filling the arrays
    unsigned int faces = coordIndex.size()/4;
    G->getC4F_N3F_V3FvertexFaceArray() = vector<C4F_N3F_V3F>(3*faces);
    vector<C4F_N3F_V3F>::iterator itVertex = G->getC4F_N3F_V3FvertexFaceArray().begin();
    
    MFInt32::const_iterator itColorIndex = colorIndex.begin();
    MFInt32::const_iterator itNormalIndex = normalIndex.begin();
    //iterate the coordIndex
    for (MFInt32::const_iterator itCoordIndex = coordIndex.begin(); itCoordIndex != coordIndex.end(); ++itCoordIndex)
    {
      if (*itCoordIndex == -1)
      {
        ++itNormalIndex;
        if (!colorPerVertex)
          ++itColorIndex;  
      }
      else
      {
        if (colorIndex.empty())
        {
          (*itVertex).color = colorRGBA[*itCoordIndex];
        }
        else
        {
          (*itVertex).color = colorRGBA[*itColorIndex];
          if (colorPerVertex)
            ++itColorIndex;  
        }
        if (normalIndex.empty())
        {
          (*itVertex).normal = normalArray[*itCoordIndex];
        }
        else
        {  
          (*itVertex).normal = normalArray[*itNormalIndex];
          if (normalPerVertex)
            ++itNormalIndex;
        }
        (*itVertex).vertex = vertexArray[*itCoordIndex];
        ++itVertex;
      }
    }
  }
}

void GLUpdaterGLGeometry3DVisitor::computeNoColor_TexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: no color, texCoord" << endl;

  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;
  
  TextureCoordinate *TexCoordNode = static_cast<TextureCoordinate *>(I->getTexCoord());
  if (TexCoordNode == 0)
    return;
  MFVec2f texCoordArray = TexCoordNode->getPoint();
 
  SFBool normalPerVertex = I->getNormalPerVertex();
  MFInt32 colorIndex = I->getColorIndex(); 
  MFInt32 normalIndex = I->getNormalIndex(); 
  MFInt32 texCoordIndex = I->getTexCoordIndex(); 
  
  //using the GL_T2F_N3F_V3F
  //vertex array
  if ((normalPerVertex) 
   && (normalIndex.empty())
   && (texCoordIndex.empty()))
  {
    //cout << "vertices not duplicated" << endl;
  
    G->getVerticesDuplicated() = false;
    //using the GL_POLYGON 
    //filling the vertex array
    MFVec2f::const_iterator itTexCoord = texCoordArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    G->getT2F_N3F_V3FvertexFaceArray() = vector<T2F_N3F_V3F>(vertexArray.size());
    vector<T2F_N3F_V3F>::iterator itElement = G->getT2F_N3F_V3FvertexFaceArray().begin();
      
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      (*itElement).texCoord.x = (*itTexCoord).x;
      (*itElement).texCoord.y = (*itTexCoord).y;      
      (*itElement).normal = *itNormal; 
      (*itElement).vertex = *itCoord;
      ++itTexCoord;
      ++itNormal;
      ++itElement;
    }
      
    //filling the index array
    //iterate the coordIndex
    unsigned int faces = coordIndex.size()/4;
    G->getIndexFaceArray() = vector<unsigned int>(3*faces);//triangles are only considered
    vector<unsigned int>::iterator itIndex = G->getIndexFaceArray().begin();
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    
    for (unsigned int i = 0; i < faces; ++i)
    {
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      *itIndex = *itCoordIndex;
      ++itCoordIndex;
      ++itIndex;
      ++itCoordIndex;//-1
    }
  }
  else //normalPerVertex is FALSE or ...
  {
    //cout << "vertices duplicated" << endl;
  
    G->getVerticesDuplicated() = true;
    //we cannot use the index array because an attribute is not constant per
    //instantiation of vertex
    //using the GL_POLYGON 
    //filling the arrays
    unsigned int faces = coordIndex.size()/4;
    G->getT2F_N3F_V3FvertexFaceArray() = vector<T2F_N3F_V3F>(3*faces);
    vector<T2F_N3F_V3F>::iterator itVertex = G->getT2F_N3F_V3FvertexFaceArray().begin();
    
    MFInt32::const_iterator itTexCoordIndex = texCoordIndex.begin();
    MFInt32::const_iterator itNormalIndex = normalIndex.begin();
    //iterate the coordIndex
    
    for (MFInt32::const_iterator itCoordIndex = coordIndex.begin(); itCoordIndex != coordIndex.end(); ++itCoordIndex)
    {
      if (*itCoordIndex == -1)
      {
        ++itTexCoordIndex;
        ++itNormalIndex;
      }  
      else
      {
        if (texCoordIndex.empty())
        {
          (*itVertex).texCoord.x = texCoordArray[*itCoordIndex].x;
          (*itVertex).texCoord.y = texCoordArray[*itCoordIndex].y; 
        }  
        else
        {
          (*itVertex).texCoord.x = texCoordArray[*itTexCoordIndex].x;
          (*itVertex).texCoord.y = texCoordArray[*itTexCoordIndex].y; 
          ++itTexCoordIndex;
        }

        if (normalIndex.empty())
          (*itVertex).normal = normalArray[*itCoordIndex];
        else
        {
          (*itVertex).normal = normalArray[*itNormalIndex];
          if (normalPerVertex) 
            ++itNormalIndex;
        }
        
        (*itVertex).vertex = vertexArray[*itCoordIndex];
        ++itVertex;
      }
    }
  }
}

void GLUpdaterGLGeometry3DVisitor::computeColor_TexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: color, texCoord" << endl;
}

void GLUpdaterGLGeometry3DVisitor::computeArrays(GLIndexedFaceSet *G, MFVec3f &vertexArray, MFVec3f &normalArray, MFInt32 &coordIndex) const
{
  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());
  
  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;

  if (!normalArray.empty())
  {
    //no process is necessary
    vertexArray = CoordNode->getPoint();
    coordIndex = I->getCoordIndex();
    return;
  }
  
  coordIndex = I->getCoordIndex();
  vertexArray = CoordNode->getPoint();
    
  SFBool normalPerVertex = I->getNormalPerVertex();
  coordIndex = I->getCoordIndex();
  
  //computing the array containing the normals of each face
  MFVec3f normalFaceArray;
  MFInt32::const_iterator itCoordIndex = coordIndex.begin();
  
  while (itCoordIndex != coordIndex.end())
  {
    SFVec3f A = vertexArray[*itCoordIndex];
    ++itCoordIndex;
    SFVec3f B = vertexArray[*itCoordIndex];
    ++itCoordIndex;
    SFVec3f C = vertexArray[*itCoordIndex];
    ++itCoordIndex;

    //computing the normal following ccw
    SFVec3f AB = (B - A).normalize();
    SFVec3f AC = (C - A).normalize();
    SFVec3f N;

    if (G->getCcw())
      N = crossprod(AB, AC);
    else
      N = crossprod(AC, AB);  

    normalFaceArray.push_back(N);
    ++itCoordIndex;//the pointed value is -1
  }

  if (normalPerVertex)
  {
    //for the moment crease angle is not used
    normalArray = MFVec3f(vertexArray.size());
    
    //iterate the faces
    itCoordIndex = coordIndex.begin();
    MFVec3f::const_iterator itNormal = normalFaceArray.begin();
    
    while (itCoordIndex != coordIndex.end())
    {
      normalArray[*itCoordIndex] = normalArray[*itCoordIndex] + *itNormal;
      ++itCoordIndex;
      normalArray[*itCoordIndex] = normalArray[*itCoordIndex] + *itNormal;
      ++itCoordIndex;
      normalArray[*itCoordIndex] = normalArray[*itCoordIndex] + *itNormal;
      ++itCoordIndex;
      ++itCoordIndex;
      ++itNormal;
    }

    //then normalize
    for (MFVec3f::iterator itNormal = normalArray.begin(); itNormal != normalArray.end(); ++itNormal)
    {
      (*itNormal).normalize();
    }
  }
  else
  {
    normalArray = MFVec3f();
    //copying the values of normalFaceArray
    for (MFVec3f::const_iterator itFace = normalFaceArray.begin(); itFace != normalFaceArray.end(); ++itFace)
      normalArray.push_back(*itFace);
  }
}

