#include "GLUpdaterGLShapeVisitor.h"
#include "GLFillProperties.h"
#include "GLLineProperties.h"
#include "GLMaterial.h"
#include "FillProperties.h"
#include "LineProperties.h"
#include "Material.h"

using namespace X3DTK;

GLUpdaterGLShapeVisitor::GLUpdaterGLShapeVisitor()
: GLShapeVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLUpdaterGLShapeVisitor, GLFillProperties>(&GLUpdaterGLShapeVisitor::enterGLFillProperties);
  defineNewEnterFunction<GLUpdaterGLShapeVisitor, GLLineProperties>(&GLUpdaterGLShapeVisitor::enterGLLineProperties);
  defineNewEnterFunction<GLUpdaterGLShapeVisitor, GLMaterial>(&GLUpdaterGLShapeVisitor::enterGLMaterial);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLUpdaterGlobalVariables>();
}

GLUpdaterGLShapeVisitor::~GLUpdaterGLShapeVisitor()
{
}

void GLUpdaterGLShapeVisitor::enterGLFillProperties(GLFillProperties *G) const
{
}

void GLUpdaterGLShapeVisitor::enterGLLineProperties(GLLineProperties *G) const
{
}

void GLUpdaterGLShapeVisitor::enterGLMaterial(GLMaterial *G) const
{
  Material *M = static_cast<Material *>(G->getLink());

  SFColor C;
  float t = 1.0f - M->getTransparency();
  float f = 1.0f - M->getAmbientIntensity();

  C = M->getDiffuseColor();
  
  G->getDiffuseColor()[0] = C.r;
  G->getDiffuseColor()[1] = C.g;
  G->getDiffuseColor()[2] = C.b;
  G->getDiffuseColor()[3] = t;

  G->getAmbientColor()[0] = f*C.r;
  G->getAmbientColor()[1] = f*C.g;
  G->getAmbientColor()[2] = f*C.b;
  G->getAmbientColor()[3] = t;

  C = M->getEmissiveColor();
  G->getEmissiveColor()[0] = C.r;
  G->getEmissiveColor()[1] = C.g;
  G->getEmissiveColor()[2] = C.b;
  G->getEmissiveColor()[3] = t;

  G->getShininess() = 128.0f*M->getShininess();  

  C = M->getSpecularColor();
  G->getSpecularColor()[0] = C.r;
  G->getSpecularColor()[1] = C.g;
  G->getSpecularColor()[2] = C.b;
  G->getSpecularColor()[3] = t;
}

